import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Analyzer extends JDialog implements Runnable
{
  private HexagonalCell hexagonalCell;
  private HexGrid hexGrid;
  private int stateMask;
  private int cells[][];
  private int columns;
  private int rows;
  
  private JPanel panel;
  private JLabel descriptionLabel;
  private JLabel periodLabel;
  private JLabel xDisplacementLabel;
  private JLabel yDisplacementLabel;

  private int c1;
  private int r1;
  private int cn;
  private int rn;

  private boolean clear;

  private int pattern[][];
  private int patternC1;
  private int patternR1;
  private int patternCn;
  private int patternRn;

  private int period;

  public Analyzer(JFrame owner, HexagonalCell hexagonalCell, HexGrid hexGrid, int stateMask)
  {
    super(owner, "Analyzer", true);
    this.hexagonalCell = hexagonalCell;
    this.hexGrid = hexGrid;
    this.stateMask = stateMask;
    panel = new JPanel();
    descriptionLabel = new JLabel("Pattern is an Oscillator");
    periodLabel = new JLabel("Period ");
    xDisplacementLabel = new JLabel("X Displacement ");
    yDisplacementLabel = new JLabel("Y Displacement ");
    panel.setBorder(BorderFactory.createLineBorder(getBackground(), 15));
    panel.setLayout(new GridLayout(4, 1, 5, 5));
    panel.add(descriptionLabel);
    panel.add(periodLabel);
    panel.add(xDisplacementLabel);
    panel.add(yDisplacementLabel);
    setLayout(new BorderLayout());
    add(panel, BorderLayout.NORTH);
    pack();
    setResizable(false);
  }

  private void getPatternLimits()
  {
    c1 = columns;
    r1 = rows;
    cn = 0;
    rn = 0;
    clear = true;
    for (int c = 0; c < columns; c++)
      for (int r = 0; r < rows; r++)
        if ((cells[c][r] & stateMask) > 0)
        {
          clear = false;
          if (r < r1)
            r1 = r;
          if (r > rn)
            rn = r;
          if (c < c1)
            c1 = c;
          if (c > cn)
            cn = c;
        }
  }

  public boolean save(int cells[][], int columns, int rows)
  {
    this.cells = cells;
    this.columns = columns;
    this.rows = rows;
    getPatternLimits();
    if (clear)
      return true;
    pattern = new int[cn - c1 + 1][rn - r1 + 1]; 
    for (int c = c1; c <= cn; c++)
      for (int r = r1; r <= rn; r++)
        pattern[c - c1][r - r1] = cells[c][r] & stateMask;
    patternC1 = c1;
    patternR1 = r1;
    patternCn = cn;
    patternRn = rn;
    period = 0;
    return false;
  }

  public boolean compare()
  {
    getPatternLimits();
    period++;
    if (((patternCn - patternC1) != (cn - c1)) || ((patternRn - patternR1) != (rn - r1)))
      return false;
    for (int c = c1; c <= cn; c++)
      for (int r = r1; r <= rn; r++)
      {
        if ((cells[c][r] & stateMask) != pattern[c - c1][r - r1])
          return false;
      }
    if ((c1 == patternC1) && (cn == patternCn) && (r1 == patternR1) && (rn == patternRn))
    {
      if (period == 1)
        descriptionLabel.setText("Pattern is Stable");
      else
        descriptionLabel.setText("Pattern is an Oscillator");
    }
    else
      descriptionLabel.setText("Pattern is a Ship");
    periodLabel.setText("Period " + period);
    xDisplacementLabel.setText("X Displacement " + (c1 - patternC1));
    yDisplacementLabel.setText("Y Displacement " + (r1 - patternR1));
    return true;
  }

  public void run()
  {
    cells = hexagonalCell.analyzerStep();
    while (! compare())
    {
      if (! hexagonalCell.isAnalyzing())
      {
        hexGrid.repaint();
        return;
      }
      cells = hexagonalCell.analyzerStep();
    }
    hexagonalCell.showAnalysisResults();
  }
}
