import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.image.BufferedImage;

class NeighborGrid extends JPanel implements MouseListener
{
  private static int gridWidth = 210;
  private static int gridHeight = 210;
  private static int gridColumns = 5;
  private static int gridRows = 5;
  private static int zoomLevel = 15;

  HexGridSpinner hexGridSpinner[][];
  private BufferedImage image;
  private int minimumWeight;
  private int maximumWeight;

  public NeighborGrid(int minimumWeight, int maximumWeight)
  {
    this.minimumWeight = minimumWeight;
    this.maximumWeight = maximumWeight;
    hexGridSpinner = new HexGridSpinner[gridColumns][gridRows];
    int spinnerWidth = 2 * zoomLevel;
    int spinnerHeight = 3 * zoomLevel;
    int rowHeight = 2 * spinnerHeight / 3;
    int xOffset = (gridWidth - (gridColumns * spinnerWidth + spinnerWidth / 2)) / 2;
    int yOffset = (gridHeight - (gridRows * rowHeight + spinnerHeight / 3)) / 2;
    for (int i = 0; i < gridColumns; i++)
      for (int j = 0; j < gridRows; j++)
      {
        int xo = xOffset;
        if (j % 2 > 0)
          xo += spinnerWidth / 2;
        if (((j == 0) || (j == 4)) && ((i == 0) || (i == 4)))
          hexGridSpinner[i][j] = null;
        else if (((j == 1) || (j == 3)) && (i == 4))
          hexGridSpinner[i][j] = null;
        else
          hexGridSpinner[i][j] = new HexGridSpinner(xo + i * spinnerWidth, yOffset + j * rowHeight,
                                                    spinnerWidth, spinnerHeight, 0);
      }
    addMouseListener(this);
    setPreferredSize(new Dimension(gridWidth, gridHeight));
  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);
    if (image == null)
      image = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_RGB);
    drawGrid();
    g.drawImage(image, 0, 0, null);
  }

  private void drawGrid()
  {
    Graphics g = image.getGraphics();
    g.setFont(new Font("Monospaced", Font.PLAIN, 14));
    g.setColor(getBackground());
    g.fillRect(0, 0, image.getWidth(), image.getHeight());
    for (int i = 0; i < gridColumns; i++)
      for (int j = 0; j < gridRows; j++)
        if (hexGridSpinner[i][j] == null)
          continue;
        else
          hexGridSpinner[i][j].draw(g);
    g.dispose();
  }

  public int getValue(int r, int c)
  {
    if (hexGridSpinner[c][r] != null)
      return hexGridSpinner[c][r].getWeight();
    else
      return 0;
  }

  public void setValue(int r, int c, int value)
  {
    if (hexGridSpinner[c][r] != null)
      hexGridSpinner[c][r].setWeight(value);
  }

  public void mousePressed(MouseEvent e)
  {
    for (int i = 0; i < gridColumns; i++)
      for (int j = 0; j < gridRows; j++)
      {
        if (hexGridSpinner[i][j] == null)
          continue;
        if (hexGridSpinner[i][j].contains(e.getX(), e.getY()))
        {
          int weight = hexGridSpinner[i][j].getWeight();
          if (e.isShiftDown())
            if (weight == minimumWeight)
              weight = maximumWeight;
            else
              weight--;
          else
            if (weight == maximumWeight)
              weight = minimumWeight;
            else
              weight++;
          hexGridSpinner[i][j].setWeight(weight);
          repaint();
          return;
        }
      }
  }

  public void mouseExited(MouseEvent e) {}
  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
}