import java.util.*;
import java.io.*;

public class RuleTable2Converter
{
  private BufferedReader in;
  private PrintWriter out;
  private ArrayList<String> list;

  private static final int noStateWeights = 16;
  private static final int neighborRows = 5;
  private static final int neighborColumns = 5;

  private int noStates;
  private int noNeighbors;
  private int noCounts;
  private int stateWeights[];
  private int neighborWeights[][];
  private int ruleTable[][];

  public static void main(String[] args)
  {
    try
    {
      File pathName = new File(args[0]);
      String[] fileNames = pathName.list();
      RuleTable2Converter converter = new RuleTable2Converter();
      for (int i = 0; i < fileNames.length; i++)
      {  
        File in = new File(pathName.getPath(), fileNames[i]);
        if (in.isDirectory())
          main(new String[] {in.getPath()});
        else if (in.getCanonicalPath().endsWith(".sqc"))
        {
          String s = in.getCanonicalPath();
          System.out.println(s);
          converter.setBufferedReader(new BufferedReader(new FileReader(in)));
          converter.readFile();
          in.delete();
          File out = new File(pathName.getPath(), fileNames[i]);
          if (! out.exists())
            out.createNewFile();
          converter.setPrintWriter(new PrintWriter(new BufferedWriter(new FileWriter(out))));
          converter.writeFile();
        }
      }
    }
    catch(IOException e)
    {  
      e.printStackTrace();
    }
  }

  public RuleTable2Converter()
  {
  }

  public void setBufferedReader(BufferedReader br)
  {
    in = br;
  }

  public void setPrintWriter(PrintWriter pw)
  {
    out = pw;
  }

  public void readFile()
  {
    try
    {
      in.readLine();
      noStates = new Integer(in.readLine().substring(10).trim());
      noNeighbors = new Integer(in.readLine().substring(13).trim());
      noCounts = new Integer(in.readLine().substring(10).trim());
      stateWeights = new int[noStateWeights];
      neighborWeights = new int[neighborRows][neighborColumns];
      ruleTable = new int[noNeighbors][noCounts];
      for (int r = 0; r < noStateWeights; r++)
        stateWeights[r] = 0;
      String values[] = in.readLine().substring(3).split(",");
      for (int i = 0; i < noStates; i++)
        stateWeights[i] = new Integer(values[i].trim());
      for (int i = 0; i < 5; i++)
      {
        values = in.readLine().substring(3).split(",");
        for (int j = 0; j < 5; j++)
          neighborWeights[i][j] = new Integer(values[j].trim());
      }
      for (int i = 0; i < noNeighbors; i++)
      {
        values = in.readLine().substring(3).split(",");
        for (int j = 0; j < noCounts; j++)
          ruleTable[i][j] = new Integer(values[j].trim());
      }
      String line;
      list = new ArrayList<String>();
      while ((line = in.readLine()) != null)
      {
        list.add(line);
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public void writeFile()
  {
    out.println("#Rule = Rule Table 2");
    out.println("#States = " + noStates);
    out.println("#Neighbors = " + noNeighbors);
    out.println("#Counts = " + noCounts);
    String s = "#SW ";
    for (int i = 0; i < noStates; i++)
      s = s + stateWeights[i] + ",";
    out.println(s.substring(0, s.lastIndexOf(",")));
    for (int i = 0; i < 5; i++)
    {
      s = "#NW ";
      for (int j = 0; j < 5; j++)
        s = s + neighborWeights[i][j] + ",";
      out.println(s.substring(0, s.lastIndexOf(",")));
    }
    for (int i = 0; i < noNeighbors; i++)
    {
      s = "#RT ";
      for (int j = 0; j < noCounts; j++)
        s = s + ruleTable[i][j] + ",";
      out.println(s.substring(0, s.lastIndexOf(",")));
    }
    for (int i = 0; i < list.size(); i++)
      out.println(list.get(i));
		out.close();
  }
}
