import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class Binary1DDialog extends JDialog implements ActionListener
{
  private int gridRows;
  private int gridColumns = 8;
  private int ruleTable[];
  private HexGridSpinner ruleGrid[][];
  private boolean changed;
  private JButton okButton;
  private JButton cancelButton;

  public Binary1DDialog(JFrame owner, int range)
  {
    super(owner, "1D Binary", true);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel ruleTablePanel = new JPanel();
    ruleTablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Table"));
    if (range == 1)
    {
      gridRows = 1;
    }
    else if (range == 2)
      gridRows = 1;
    else if (range == 3)
      gridRows = 4;
    else
      gridRows = 16;
    ruleTablePanel.setLayout(new GridLayout(gridRows, gridColumns));
    ruleGrid = new HexGridSpinner[gridRows][gridColumns];
    for (int r = 0; r < gridRows; r++)
      for (int c = gridColumns - 1; c >= 0; c--)
      {
        ruleGrid[r][c] = new HexGridSpinner(20, 20, 0, 0, 15);
        ruleTablePanel.add(ruleGrid[r][c]);
        if (range == 1)
          if ((r == 0) && (c < 2))
            ruleGrid[r][c].setSpinnerEnabled(true);
          else
            ruleGrid[r][c].setSpinnerEnabled(false);
      }
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(ruleTablePanel, BorderLayout.NORTH);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  public void setData(Binary1DRule rule)
  {
    ruleTable = rule.getRuleTable();
    int ci = gridColumns - 1;
    if (ruleTable.length == 2)
      ci = 1;
    for (int i = ruleTable.length - 1, r = 0, c = ci; i >= 0; i--)
    {
      ruleGrid[r][c].setValue(ruleTable[i]);
      c--;
      if (c < 0)
      {
        r++;
        c = gridColumns - 1;
      }
    }
  }

  public void getData(Binary1DRule rule, int range)
  {
    int length = (int)Math.pow(2, 2 * range - 1);
    ruleTable = new int[length];
    int ci = gridColumns - 1;
    if (length == 2)
      ci = 1;
    for (int i = 0, r = 0, c = ci; i < length; i++)
    {
      ruleTable[i] = ruleGrid[r][c].getValue();
      c--;
      if (c < 0)
      {
        r++;
        c = gridColumns - 1;
      }
    }
    rule.setRuleTable(ruleTable);
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}
