import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class Binary1DOptionsDialog extends JDialog implements ActionListener, ChangeListener
{
  private int range;
  private int ruleDensity;
  private boolean randomInitialization = true;
  private boolean changed;
  private JLabel rangeLabel;
  private SpinnerNumberModel rangeModel;
  private JSpinner rangeInput;
  private JLabel ruleDensityLabel;
  private SpinnerNumberModel ruleDensityModel;
  private JSpinner ruleDensityInput;
  private ButtonGroup patternSelector;
  private JRadioButton randomPatternButton;
  private JRadioButton pointPatternButton;
  private JButton okButton;
  private JButton cancelButton;

  public Binary1DOptionsDialog(JFrame owner)
  {
    super(owner, "1D Binary Options", true);
    rangeLabel = new JLabel("Range");
    rangeModel = new SpinnerNumberModel(2, 1, 4, 1);
    rangeInput = new JSpinner(rangeModel);
    rangeInput.addChangeListener(this);
    ruleDensityLabel = new JLabel("Rule Density %");
    ruleDensityModel = new SpinnerNumberModel(50, 1, 100, 5);
    ruleDensityInput = new JSpinner(ruleDensityModel);
    ruleDensityInput.addChangeListener(this);
    patternSelector = new ButtonGroup();
    randomPatternButton = new JRadioButton("Random Initialization");
    randomPatternButton.addActionListener(this);
    randomPatternButton.setSelected(true);
    patternSelector.add(randomPatternButton);
    pointPatternButton = new JRadioButton("Point Initialization");
    pointPatternButton.addActionListener(this);
    patternSelector.add(pointPatternButton);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel ruleRangePanel = new JPanel();
    ruleRangePanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel rangeInputPanel = new JPanel();
    ruleRangePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Table Size"));
    rangeInputPanel.add(rangeInput);
    ruleRangePanel.add(rangeLabel);
    ruleRangePanel.add(rangeInputPanel);
    JPanel newRuleOptionsPanel = new JPanel();
    newRuleOptionsPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "New Rule Options"));
    newRuleOptionsPanel.setLayout(new BorderLayout(0, 5));
    JPanel ruleDensityPanel = new JPanel();
    ruleDensityPanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel ruleDensityInputPanel = new JPanel();
    ruleDensityInputPanel.add(ruleDensityInput);
    ruleDensityPanel.add(ruleDensityLabel);
    ruleDensityPanel.add(ruleDensityInputPanel);
    newRuleOptionsPanel.add(ruleDensityPanel, BorderLayout.NORTH);
    JPanel patternSelectorPanel = new JPanel();
    patternSelectorPanel.setLayout(new GridLayout(2, 1, 5, 5));
    patternSelectorPanel.add(randomPatternButton);
    patternSelectorPanel.add(pointPatternButton);
    newRuleOptionsPanel.add(patternSelectorPanel, BorderLayout.SOUTH);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(ruleRangePanel, BorderLayout.NORTH);
    basePanel.add(newRuleOptionsPanel, BorderLayout.CENTER);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  private void setValues()
  {
    range = rangeModel.getNumber().intValue();
    ruleDensity = ruleDensityModel.getNumber().intValue();
  }

  public void setData(Binary1DRule rule)
  {
    range = rule.getRange();
    rangeModel.setValue(range);
    ruleDensity = rule.getRuleDensity();
    ruleDensityModel.setValue(ruleDensity);
    randomInitialization = rule.getRandomInitialization();
    if (randomInitialization)
      randomPatternButton.setSelected(true);
    else
      pointPatternButton.setSelected(true);
  }

  public void getData(Binary1DRule rule)
  {
    rule.setRange(range);
    rule.setRuleDensity(ruleDensity);
    rule.setRandomInitialization(randomInitialization);
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == randomPatternButton)
      randomInitialization = true;
    else if (source == pointPatternButton)
      randomInitialization = false;
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void stateChanged(ChangeEvent event)
  {
    setValues();
  }
}
