import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

class Binary1DRule extends Rule
{
  private int range;
  private int ruleTable[];
  private int hexRuleTable[];
  private int ruleDensity = 60;
  private boolean randomInitialization = true;

  public Binary1DRule(JFrame mainFrame, SquareCell1D squareCell1D)
  {
    super(mainFrame, squareCell1D);
    this.mainFrame = mainFrame;
    this.squareCell1D = squareCell1D;
    setDefault();
  }

  public boolean setNewRule()
  {
    for (int i = 0; i < ruleTable.length; i++)
      ruleTable[i] = 0;
    Random generator = new Random();
    for (int i = 0; i < ruleTable.length * ruleDensity / 100; i++)
      ruleTable[generator.nextInt(ruleTable.length)] = 1;
    makeHexRuleTable();
    return randomInitialization;
  }

  private void makeRuleTable(int rt[])
  {
    ruleTable = new int[rt.length * 4];
    int j = 0;
    for (int i = rt.length - 1; i >= 0; i--)
    {
      ruleTable[j++] = rt[i] & 1;
      ruleTable[j++] = (rt[i] >> 1) & 1;
      ruleTable[j++] = (rt[i] >> 2) & 1;
      ruleTable[j++] = (rt[i] >> 3) & 1;
    }
    makeHexRuleTable();
  }

  private void makeHexRuleTable()
  {
    int length = (int)Math.pow(2, 2 * range - 1);
    hexRuleTable = new int[length];
    for (int i = ruleTable.length - 4, j = length - 1; i >= 0; i -= 4, j--)
      hexRuleTable[j] = (ruleTable[i + 3] << 3) + (ruleTable[i + 2] << 2) + (ruleTable[i + 1] << 1) + ruleTable[i];
  }

  public void setDefault()
  {
    range = 2;
    final int rt[] = {8, 9, 0xE, 0xD, 7, 1, 0, 6};
    makeRuleTable(rt);
  }

  public void setRule110()
  {
    range = 1;
    final int rt[] = {6, 0xE};
    makeRuleTable(rt);
  }

  public void changeRule()
  {
    Binary1DDialog binary1DDialog = new Binary1DDialog(mainFrame, range);
    binary1DDialog.setData(this);
    centerDialog(binary1DDialog);
    binary1DDialog.setVisible(true);
    if (binary1DDialog.hasChanged())
      binary1DDialog.getData(this, range);
  }

  public void setOptions()
  {
    Binary1DOptionsDialog binary1DOptionsDialog = new Binary1DOptionsDialog(mainFrame);
    binary1DOptionsDialog.setData(this);
    centerDialog(binary1DOptionsDialog);
    binary1DOptionsDialog.setVisible(true);
    if (binary1DOptionsDialog.hasChanged())
      binary1DOptionsDialog.getData(this);
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  public void writeRule(PrintWriter pw)
  {
    pw.println("#Rule = 1D Binary");
    pw.println("#Range = " + range);
    String s ="";
    for (int i = hexRuleTable.length - 1; i >= 0; i--)
      s = s + Integer.toHexString(hexRuleTable[i]);
    pw.println("#RT = " + s);
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      range = new Integer(br.readLine().substring(9).trim());
      String s = br.readLine().substring(6).trim();
      int length = (int)Math.pow(2, 2 * range - 1);
      int rt[] = new int[length];
      for (int i = length - 1; i >= 0; i--)
        rt[i] = Integer.parseInt(s.substring(i, i + 1), 16);
      makeRuleTable(rt);
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public int step(int row, int column)
  {
    int index = 0;
    for (int i = 0, j = range; i < (range * 2 + 1); i++, j--)
      index += (squareCell1D.getNeighbor(row, column + j) * Math.pow(2, i));
    return ruleTable[index];
  }

  public int getRange()
  {
    return range;
  }

  public void setRange(int range)
  {
    if (this.range != range)
    {
      this.range = range;
      ruleTable = new int[(int)Math.pow(2, 2 * range + 1)];
    }
  }

  public int[] getRuleTable()
  {
    return hexRuleTable;
  }

  public void setRuleTable(int[] ruleTable)
  {
    makeRuleTable(ruleTable);
  }

  public int getRuleDensity()
  {
    return ruleDensity;
  }

  public void setRuleDensity(int ruleDensity)
  {
    this.ruleDensity = ruleDensity;
  }

  public boolean getRandomInitialization()
  {
    return randomInitialization;
  }

  public void setRandomInitialization(boolean randomInitialization)
  {
    this.randomInitialization = randomInitialization;
  }

  public int getNoStates()
  {
    return 2;
  }
}
