import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class DavidEckOptionsDialog extends JDialog implements ActionListener
{
  private int noStates;
  private int noNeighbors;
  private boolean isotropic;
  private boolean randomInitialization = true;

  private boolean changed;

  private JLabel noStatesLabel;
  private JLabel noNeighborsLabel;
  private JComboBox noStatesSelector;
  private JComboBox noNeighborsSelector;
  private JCheckBox isotropicButton;
  private ButtonGroup patternSelector;
  private JRadioButton randomPatternButton;
  private JRadioButton pointPatternButton;
  private JButton okButton;
  private JButton cancelButton;

  public DavidEckOptionsDialog(JFrame owner)
  {
    super(owner, "David Eck Options", true);
    noStatesLabel = new JLabel("No States");
    noNeighborsLabel = new JLabel("No Neighbors");
    String stateOptions[] = {"2", "3", "4", "5", "6", "7", "8", "9"};
    noStatesSelector = new JComboBox(stateOptions);
    noStatesSelector.addActionListener(this);
    String neighborOptions[] = {"3", "5", "7", "9"};
    noNeighborsSelector = new JComboBox(neighborOptions);
    noNeighborsSelector.addActionListener(this);
    isotropicButton = new JCheckBox("Isotropic");
    isotropicButton.setSelected(true);
    patternSelector = new ButtonGroup();
    randomPatternButton = new JRadioButton("Random");
    randomPatternButton.addActionListener(this);
    randomPatternButton.setSelected(true);
    patternSelector.add(randomPatternButton);
    pointPatternButton = new JRadioButton("Point");
    pointPatternButton.addActionListener(this);
    patternSelector.add(pointPatternButton);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel ruleTableSizePanel = new JPanel();
    ruleTableSizePanel.setLayout(new GridLayout(2, 2, 5, 5));
    ruleTableSizePanel.add(noStatesLabel);
    JPanel statesSelector = new JPanel();
    statesSelector.add(noStatesSelector);
    ruleTableSizePanel.add(statesSelector);
    ruleTableSizePanel.add(noNeighborsLabel);
    JPanel neighborsSelector = new JPanel();
    neighborsSelector.add(noNeighborsSelector);
    ruleTableSizePanel.add(neighborsSelector);
    JPanel ruleOptionsPanel = new JPanel();
    ruleOptionsPanel.setLayout(new BorderLayout(0, 5));
    ruleOptionsPanel.add(ruleTableSizePanel, BorderLayout.NORTH);
    ruleOptionsPanel.add(isotropicButton, BorderLayout.SOUTH);
    ruleOptionsPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Options"));
    JPanel patternSelectorPanel = new JPanel();
    patternSelectorPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Pattern Initialization"));
    patternSelectorPanel.setLayout(new GridLayout(2, 1, 5, 5));
    patternSelectorPanel.add(randomPatternButton);
    patternSelectorPanel.add(pointPatternButton);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(ruleOptionsPanel, BorderLayout.NORTH);
    basePanel.add(patternSelectorPanel, BorderLayout.CENTER);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  private void setValues()
  {
    try
    {
      noStates = new Integer((String)noStatesSelector.getSelectedItem());
      noNeighbors = new Integer((String)noNeighborsSelector.getSelectedItem());
      int maxNeighbors = 0;
      if ((noStates == 2) || (noStates == 3))
        maxNeighbors = 9;
      else if (noStates == 4)
        maxNeighbors = 7;
      else if (noStates > 7)
        maxNeighbors = 3;
      else
        maxNeighbors = 5;
      if (noNeighbors > maxNeighbors)
      {
        noNeighbors = maxNeighbors;
        noNeighborsSelector.setSelectedItem("" + noNeighbors);
      }
    }
    catch (Exception e)
    {
    }
  }

  public void setData(DavidEckRule rule)
  {
    noStates = rule.getNoStates();
    noStatesSelector.setSelectedItem("" + noStates);
    noNeighbors = rule.getNoNeighbors();
    noNeighborsSelector.setSelectedItem("" + noNeighbors);
    isotropicButton.setSelected(rule.getIsotropic());
    randomInitialization = rule.getRandomInitialization();
    if (randomInitialization)
      randomPatternButton.setSelected(true);
    else
      pointPatternButton.setSelected(true);
  }

  public void getData(DavidEckRule rule)
  {
    rule.setNoStates(noStates);
    rule.setNoNeighbors(noNeighbors);
    rule.setIsotropic(isotropicButton.isSelected());
    rule.setRandomInitialization(randomInitialization);
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == randomPatternButton)
      randomInitialization = true;
    else if ((source == noStatesSelector) || (source == noNeighborsSelector))
      setValues();
    else if (source == pointPatternButton)
      randomInitialization = false;
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}
