import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

public class DavidEckRule extends Rule
{
  private int states;
  private int neighbors;
  private int ruleCt;
  private int lambdaCt;
  private int newLambdaCt;
  private int isoStates;
  private int isoNeighbors;
  private int isoCt;
  private int rules[];
  private int lambdaRule[];
  private int lambdaState[];
  private int isoMate[];
  private boolean isotropic;
  private boolean randomInitialization;
  private Random rand;

  public DavidEckRule(JFrame mainFrame, SquareCell1D squareCell1D)
  {
    super(mainFrame, squareCell1D);
    this.mainFrame = mainFrame;
    this.squareCell1D = squareCell1D;
    rand = new Random();
    setDefault();
  }

  public void setDefault()
  {
    states = 4;
    neighbors = 5;
    isotropic = true;
    randomInitialization = true;
    doNew();
  }

  private void doNew()
  {
    ruleCt = 1;
    for (int i = 1; i <= neighbors; i++)
      ruleCt = ruleCt * states;
    rules = new int[ruleCt];
    if (!isotropic)
    {
      lambdaRule = new int[ruleCt];
      lambdaState = new int[ruleCt];
      makeRules();
    }
    else
    {
      if (isoStates != states || isoNeighbors != neighbors)
        computeisoMate();
      lambdaRule = new int[isoCt];
      lambdaState = new int[isoCt];
      makeIsoRules();
      ruleCt = isoCt;
    }
    newLambdaCt = ruleCt / 2;
    newLambda();
/*
    System.out.println("states = " + states + ", neighbors = " + neighbors + ", isotropic = " + (isotropic ? "true" : "false"));
    System.out.println("ruleCt = " + ruleCt + ", lambdaCt = " + lambdaCt);
    System.out.println("rules length = " + rules.length +
                       ", lambdaRule length " + lambdaRule.length +
                       ", lambdaState length = " + lambdaState.length);
    System.out.println();
*/                   
  }

  private void makeRules()
  {
    rules[0] = 0;
    lambdaState[0] = 0;
    lambdaRule[0] = 0;
    for (int i = 1; i < ruleCt; i++)
    {
      rules[i] = 0;
      lambdaRule[i] = i;
      lambdaState[i] = 1 + (int)(rand.nextDouble() * (states - 1));
      if (lambdaState[i] >= states)
        lambdaState[i] = states;
    }
    for (int i = ruleCt - 1; i > 1; i--)
    {
      int r = 1 + (int)(rand.nextDouble() * i);
      if (r > i)
        r = i;
      int temp = lambdaRule[i];
      lambdaRule[i] = lambdaRule[r];
      lambdaRule[r] = temp;
    }
    lambdaCt = 0;
  }

  private void makeIsoRules()
  {
    boolean[] used = new boolean[ruleCt];
    for (int i = 0; i < ruleCt; i++)
    {
      used[i] = false;
      rules[i] = 0;
    }
    lambdaState[0] = 0;
    lambdaRule[0] = 0;
    int k = 1;
    for (int i = 1; i < ruleCt; i++)
    {
      if (!used[i])
      {
        lambdaRule[k] = i;
        used[isoMate[i]] = true;   
        k++;
      }
    }
    for (int i = isoCt - 1; i > 1; i--)
    {
      int r = 1 + (int)(rand.nextDouble() * i);
      if (r > i)
        r = i;
      int temp = lambdaRule[i];
      lambdaRule[i] = lambdaRule[r];
      lambdaRule[r] = temp;
    }
    for (int i = 1; i < isoCt; i++)
    {
      lambdaState[i] = 1 + (int)(rand.nextDouble() * (states - 1));
      if (lambdaState[i] >= states)
        lambdaState[i] = states;
    }
    lambdaCt = 0;
  }

  private void newLambda()
  {
    if (!isotropic)
    {
      if (newLambdaCt > lambdaCt)
      {
        for (int i = lambdaCt + 1; i <= newLambdaCt; i++)
          setRule(lambdaRule[i], lambdaState[i]);
      }
      else if (newLambdaCt < lambdaCt)
      {
        for (int i = lambdaCt; i > newLambdaCt; i--)
          setRule(lambdaRule[i], 0);
      }
    }
    else
    {
      if (newLambdaCt > lambdaCt)
      {
        for (int i = lambdaCt + 1; i <= newLambdaCt; i++)
        {
          setRule(lambdaRule[i], lambdaState[i]);
          setRule(isoMate[lambdaRule[i]], lambdaState[i]);
        }
      }
      else if (newLambdaCt < lambdaCt)
      {
        for (int i = lambdaCt; i > newLambdaCt; i--)
        {
          setRule(lambdaRule[i], 0);
          setRule(isoMate[lambdaRule[i]], 0);
        }
      }
    }
    lambdaCt = newLambdaCt;
  }

  private void setRule(int ruleNum, int value)
  {
    if ((rules != null) && (value >= 0) && (value < states) && (ruleNum >= 0) && (ruleNum < rules.length))
      rules[ruleNum] = value;
  }

  private void computeisoMate()
  {
    if (isoMate == null || isoMate.length < ruleCt)
      isoMate = new int[ruleCt];
    isoMate[0] = 0;
    for (int i = 1; i < ruleCt; i++)
      isoMate[i] = -1;
    int[] num = new int[9];
    for (int i = 0; i <= 8; i++)
       num[i] = 0;
    isoCt = 1;
    int y = 0;
    do
    {
      int j = 0;
      while (j < neighbors && ++num[j] >= states)
      {
        num[j] = 0;
        j++;
      }
      y++;
      if (isoMate[y] == -1)
      {
        int x = num[0];
        for (int k = 1; k < neighbors; k++)
          x = states * x + num[k];
        isoMate[x] = y;
        isoMate[y] = x;
        isoCt++;
      }
    }
    while (y < ruleCt - 1);
    isoStates = states;
    isoNeighbors = neighbors;
  }

  public boolean setNewRule()
  {
    doNew();
    return randomInitialization;
  }

  public int getLambdaCt()
  {
    return lambdaCt * 1000 / ruleCt;
  }

  public boolean incLambdaCt(int increment)
  {
    newLambdaCt += increment;
    if (newLambdaCt >= ruleCt)
      newLambdaCt = ruleCt - 1;
    newLambda();
    return randomInitialization;
  }

  public boolean decLambdaCt(int increment)
  {
    newLambdaCt -= increment;
    if (newLambdaCt < 0)
      newLambdaCt = 0;
    newLambda();
    return randomInitialization;
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  public void setOptions()
  {
    DavidEckOptionsDialog optionsDialog = new DavidEckOptionsDialog(mainFrame);
    optionsDialog.setData(this);
    centerDialog(optionsDialog);
    optionsDialog.setVisible(true);
    if (optionsDialog.hasChanged())
    {
      optionsDialog.getData(this);
      squareCell1D.generateAndRunNewRule();
    }
  }

  public void changeRule()
  {
  }

  public void writeRule(PrintWriter pw)
  {
    pw.println("#Rule = David Eck");
    pw.println("#States = " + states);
    pw.println("#Neighbors = " + neighbors);
    pw.println("#Isotropic = " + (isotropic ? "true" : "false"));
    pw.println("#Rule Count = " + ruleCt);
    pw.println("#Lambda Count = " + lambdaCt);
    pw.println("#Rules");
    printState(pw, rules);
    pw.println("#Lambda Rules");
    printIndex(pw, lambdaRule);
    pw.println("#Lambda States");
    printState(pw, lambdaState);
  }

  private void printState(PrintWriter pw, int[] a)
  {
    pw.print("#Length = " + a.length);
    for (int i = 0; i < a.length; i++)
    {
      if ((i % 100) == 0)
      {
        pw.println();
        pw.print("#T ");
      }
      pw.print(a[i]);
    }
    pw.println();
  }

  private void printIndex(PrintWriter pw, int[] a)
  {
    pw.print("#Length = " + a.length);
    for (int i = 0; i < a.length; i++)
    {
      if ((i % 25) == 0)
      {
        pw.println();
        pw.print("#T ");
      }
      pw.print(a[i] + ",");
    }
    pw.println();
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      states = new Integer(br.readLine().substring(10).trim());
      neighbors = new Integer(br.readLine().substring(13).trim());
      if (br.readLine().substring(13).trim().equals("true"))
        isotropic = true;
      else
        isotropic = false;
      ruleCt = new Integer(br.readLine().substring(14).trim());
      lambdaCt = new Integer(br.readLine().substring(16).trim());
      String line = br.readLine();
      int length = new Integer(br.readLine().substring(10).trim());
      rules = new int[length];
      readState(br, length, rules); 
      line = br.readLine();
      length = new Integer(br.readLine().substring(10).trim());
      lambdaRule = new int[length];
      readIndex(br, length, lambdaRule); 
      line = br.readLine();
      length = new Integer(br.readLine().substring(10).trim());
      lambdaState = new int[length];
      readState(br, length, lambdaState); 
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  private void readState(BufferedReader br, int length, int a[])
  {
    try
    {
      String line = "";
      for (int i = 0; i < length; i++)
      {
        if ((i % 100) == 0)
          line = br.readLine().substring(3);
        a[i] = new Integer(line.substring(0, 1));
        line = line.substring(1);
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }
  private void readIndex(BufferedReader br, int length, int a[])
  {
    try
    {
      String line = "";
      for (int i = 0; i < length; i++)
      {
        if ((i % 25) == 0)
          line = br.readLine().substring(3);
        int end = line.indexOf(",");
        a[i] = new Integer(line.substring(0, end));
        line = line.substring(end + 1);
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
    }
  }

  public int step(int row, int column)
  {
    int index = 0;
    for (int i = column - neighbors / 2; i <= column + neighbors / 2; i++)
      index = states * index + squareCell1D.getNeighbor(row, i);
    return rules[index];
  }

  public int getNoStates()
  {
    return states;
  }

  public int getNoNeighbors()
  {
    return neighbors;
  }

  public boolean getIsotropic()
  {
    return isotropic;
  }

  public boolean getRandomInitialization()
  {
    return randomInitialization;
  }

  public void setNoStates(int noStates)
  {
    states = noStates;
  }

  public void setNoNeighbors(int noNeighbors)
  {
    neighbors = noNeighbors;
  }

  public void setIsotropic(boolean isotropic)
  {
    this.isotropic = isotropic;
  }

  public void setRandomInitialization(boolean randomInitialization)
  {
    this.randomInitialization = randomInitialization;
  }
}
