import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class NewWorld extends JDialog implements ActionListener, DocumentListener
{
  private int rows;
  private int columns;
  private String selectedRule;

  private boolean changed;

  private JLabel rowsLabel;
  private JLabel columnsLabel;
  private JTextField rowsInput;
  private JTextField columnsInput;
  private ButtonGroup ruleSelector;
  private JRadioButton binary1DButton;
  private JRadioButton ruleTable1DButton;
  private JRadioButton ruleTable21DButton;
  private JRadioButton DavidEckButton;
  private JButton okButton;
  private JButton cancelButton;

  public NewWorld(JFrame owner, int rows, int columns, String selectedRule)
  {
    super(owner, "New World", true);
    this.rows = rows;
    this.columns = columns;
    this.selectedRule = selectedRule;
    rowsLabel = new JLabel("Rows");
    columnsLabel = new JLabel("Columns");
    rowsInput = new JTextField("" + rows, 4);
    rowsInput.getDocument().addDocumentListener(this);
    columnsInput = new JTextField("" + columns, 4);
    columnsInput.getDocument().addDocumentListener(this);
    ruleSelector = new ButtonGroup();
    binary1DButton = new JRadioButton("1D Binary");
    binary1DButton.addActionListener(this);
    ruleSelector.add(binary1DButton);
    ruleTable1DButton = new JRadioButton("1D Rule Table");
    ruleTable1DButton.addActionListener(this);
    ruleSelector.add(ruleTable1DButton);
    ruleTable21DButton = new JRadioButton("1D Rule Table 2");
    ruleTable21DButton.addActionListener(this);
    ruleSelector.add(ruleTable21DButton);
    DavidEckButton = new JRadioButton("David Eck");
    DavidEckButton.addActionListener(this);
    ruleSelector.add(DavidEckButton);
    if (selectedRule.equals("1D Binary"))
      binary1DButton.setSelected(true);
    else if (selectedRule.equals("1D Rule Table"))
      ruleTable1DButton.setSelected(true);
    else if (selectedRule.equals("1D Rule Table 2"))
      ruleTable21DButton.setSelected(true);
    else if (selectedRule.equals("David Eck"))
      DavidEckButton.setSelected(true);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel worldSizePanel = new JPanel();
    worldSizePanel.setLayout(new GridLayout(2, 2, 5, 5));
    JPanel rowsInputPanel = new JPanel();
    rowsInputPanel.add(rowsInput);
    JPanel columnsInputPanel = new JPanel();
    columnsInputPanel.add(columnsInput);
    worldSizePanel.add(rowsLabel);
    worldSizePanel.add(rowsInputPanel);
    worldSizePanel.add(columnsLabel);
    worldSizePanel.add(columnsInputPanel);
    worldSizePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "World Size"));
    JPanel ruleSelectorPanel = new JPanel();
    ruleSelectorPanel.setLayout(new GridLayout(4, 1, 5, 5));
    ruleSelectorPanel.add(binary1DButton);
    ruleSelectorPanel.add(ruleTable1DButton);
    ruleSelectorPanel.add(ruleTable21DButton);
    ruleSelectorPanel.add(DavidEckButton);
    ruleSelectorPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Selector"));
    JPanel optionsPanel = new JPanel();
    optionsPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 5));
    optionsPanel.add(worldSizePanel);
    optionsPanel.add(ruleSelectorPanel);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(optionsPanel, BorderLayout.NORTH);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
    setValues();
  }

  private void setValues()
  {
    try
    {
      rows = new Integer(rowsInput.getText().trim());
      columns = new Integer(columnsInput.getText().trim());
      if (rows < 1)
        rows = 1;
      if (columns < 10)
        columns = 10;
    }
    catch (Exception e)
    {
    }
  }

  public int getRows()
  {
    return rows;
  }

  public int getColumns()
  {
    return columns;
  }

  public String getSelectedRule()
  {
    return selectedRule;
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == binary1DButton)
      selectedRule = "1D Binary";
    else if (source == ruleTable1DButton)
      selectedRule = "1D Rule Table";
    else if (source == ruleTable21DButton)
      selectedRule = "1D Rule Table 2";
    else if (source == DavidEckButton)
      selectedRule = "David Eck";
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
