import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

class RuleTable21DRule extends Rule
{
  private static final int noStateWeights = 16;
  private static final int neighborRows = 5;
  private static final int neighborColumns = 5;

  private int noStates;
  private int noCounts;
  private int percentChange = 60;
  private String newRuleOption = "Random";
  private boolean clearNewRule = true;
  private int stateWeights[];
  private int neighborWeights[][];
  private int ruleTable[][];
  private int useTable[][];
  private boolean randomInitialization = true;

  public RuleTable21DRule(JFrame mainFrame, SquareCell1D squareCell1D)
  {
    super(mainFrame, squareCell1D);
    setDefault();
  }

  private void makeArrays()
  {
    stateWeights = new int[noStateWeights];
    neighborWeights = new int[neighborRows][neighborColumns];
    ruleTable = new int[noStates][noCounts];
    useTable = new int[noStates][noCounts];
  }

  private void setWeights()
  {
    for (int r = 0; r < noStateWeights; r++)
      stateWeights[r] = 0;
    stateWeights[1] = 1;
    for (int r = 0; r < neighborRows; r++)
      for (int c = 0; c < neighborColumns; c++)
        if (r == 2)
          neighborWeights[r][c] = 1;
        else
          neighborWeights[r][c] = 0;
  }

  public void setDefault()
  {
    noStates = 2;
    noCounts = 9;
    makeArrays();
    setWeights();
    for (int r = 0; r < noStates; r++)
      for (int c = 0; c < noCounts; c++)
          ruleTable[r][c] = 0;
  }

  public void changeRule()
  {
    RuleTable21DDialog RuleTable21DDialog = new RuleTable21DDialog(mainFrame, this, squareCell1D);
    RuleTable21DDialog.setData(this);
    centerDialog(RuleTable21DDialog);
    RuleTable21DDialog.setVisible(true);
    if (RuleTable21DDialog.hasChanged())
		{
      stateWeights = new int[noStateWeights];
      RuleTable21DDialog.getData(this);
		}
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  private void clearUseTable()
  {
    for (int r = 0; r < noStates; r++)
      for (int c = 0; c < noCounts; c++)
        useTable[r][c] = 0;
  }

  public boolean setNewRule()
  {
    random(ruleTable);
    return randomInitialization;
  }

  public void random(int ruleTable[][])
  {
    clearUseTable();
    if (clearNewRule)
    {
      for (int r = 0; r < noStates; r++)
        for (int c = 0; c < noCounts; c++)
          ruleTable[r][c] = 0;
    }
    Random generator = new Random();
    if (newRuleOption.equals("Random"))
    {
      for (int i = 1; i < noStates * noCounts * percentChange / 100; i++)
      {
        int r = generator.nextInt(noStates);
        int c = generator.nextInt(noCounts);
        ruleTable[r][c] = generator.nextInt(noStates);
      }
    }
    else if (newRuleOption.equals("Generations") || newRuleOption.equals("Modified Generations"))
    {
      for (int r = 1; r < (noStates - 1); r++)
        for (int c = 0; c < noCounts; c++)
          ruleTable[r][c] = r + 1;
      if (newRuleOption.equals("Generations"))
        for (int i = 1; i < 2 * noCounts * percentChange / 100; i++)
        {
          int r = generator.nextInt(2);
          int c = generator.nextInt(noCounts);
          ruleTable[r][c] = 1;
        }
      else
        for (int i = 1; i < noStates * noCounts * percentChange / 100; i++)
        {
          int r = generator.nextInt(noStates);
          int c = generator.nextInt(noCounts);
          ruleTable[r][c] = generator.nextInt(noStates);
        }
    }
  }

  public void setOptions()
  {
    int currentNoStates = noStates;
    int currentNoCounts = noCounts;
    RuleTable21DOptionsDialog RuleTable21DOptionsDialog = new RuleTable21DOptionsDialog(mainFrame);
    RuleTable21DOptionsDialog.setData(this);
    centerDialog(RuleTable21DOptionsDialog);
    RuleTable21DOptionsDialog.setVisible(true);
    if (RuleTable21DOptionsDialog.hasChanged())
    {
      RuleTable21DOptionsDialog.getData(this);
      if ((noStates != currentNoStates) || (noCounts != currentNoCounts))
      {
				if (noStates != currentNoStates)
					for (int i = noStates; i < noStateWeights; i++)
					  stateWeights[i] = 0;
        ruleTable = new int[noStates][noCounts];
        useTable = new int[noStates][noCounts];
      }
    }
  }

  public void writeRule(PrintWriter pw)
  {
    pw.println("#Rule = 1D Rule Table 2");
    pw.println("#States = " + noStates);
    pw.println("#Counts = " + noCounts);
    String s = "#SW ";
    for (int i = 0; i < noStates; i++)
      s = s + stateWeights[i] + ",";
    pw.println(s.substring(0, s.lastIndexOf(",")));
    for (int i = 0; i < 5; i++)
    {
      s = "#NW ";
      for (int j = 0; j < 5; j++)
        s = s + neighborWeights[i][j] + ",";
      pw.println(s.substring(0, s.lastIndexOf(",")));
    }
    for (int i = 0; i < noStates; i++)
    {
      s = "#RT ";
      for (int j = 0; j < noCounts; j++)
        s = s + ruleTable[i][j] + ",";
      pw.println(s.substring(0, s.lastIndexOf(",")));
    }
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      noStates = new Integer(br.readLine().substring(10).trim());
      noCounts = new Integer(br.readLine().substring(10).trim());
      ruleTable = new int[noStates][noCounts];
      useTable = new int[noStates][noCounts];
			for (int i = 0; i < noStateWeights; i++)
				stateWeights[i] = 0;
      String values[] = br.readLine().substring(3).split(",");
      for (int i = 0; i < noStates; i++)
        stateWeights[i] = new Integer(values[i].trim());
      for (int i = 0; i < 5; i++)
      {
        values = br.readLine().substring(3).split(",");
        for (int j = 0; j < 5; j++)
          neighborWeights[i][j] = new Integer(values[j].trim());
      }
      for (int i = 0; i < noStates; i++)
      {
        values = br.readLine().substring(3).split(",");
        for (int j = 0; j < noCounts; j++)
          ruleTable[i][j] = new Integer(values[j].trim());
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  public int step(int r, int c)
  {
    int cell = squareCell1D.getNeighbor(r, c);
    int count =
      stateWeights[squareCell1D.getNeighbor(r - 4, c - 2)] * neighborWeights[0][0] +
      stateWeights[squareCell1D.getNeighbor(r - 4, c - 1)] * neighborWeights[0][1] +
      stateWeights[squareCell1D.getNeighbor(r - 4, c    )] * neighborWeights[0][2] +
      stateWeights[squareCell1D.getNeighbor(r - 4, c + 1)] * neighborWeights[0][3] +
      stateWeights[squareCell1D.getNeighbor(r - 4, c + 2)] * neighborWeights[0][4] +

      stateWeights[squareCell1D.getNeighbor(r - 3, c - 2)] * neighborWeights[1][0] +
      stateWeights[squareCell1D.getNeighbor(r - 3, c - 1)] * neighborWeights[1][1] +
      stateWeights[squareCell1D.getNeighbor(r - 3, c    )] * neighborWeights[1][2] +
      stateWeights[squareCell1D.getNeighbor(r - 3, c + 1)] * neighborWeights[1][3] +
      stateWeights[squareCell1D.getNeighbor(r - 3, c + 2)] * neighborWeights[1][4] +

      stateWeights[squareCell1D.getNeighbor(r - 2, c - 2)] * neighborWeights[2][0] +
      stateWeights[squareCell1D.getNeighbor(r - 2, c - 1)] * neighborWeights[2][1] +
      stateWeights[squareCell1D.getNeighbor(r - 2, c    )] * neighborWeights[2][2] +
      stateWeights[squareCell1D.getNeighbor(r - 2, c + 1)] * neighborWeights[2][3] +
      stateWeights[squareCell1D.getNeighbor(r - 2, c + 2)] * neighborWeights[2][4] +

      stateWeights[squareCell1D.getNeighbor(r - 1, c - 2)] * neighborWeights[3][0] +
      stateWeights[squareCell1D.getNeighbor(r - 1, c - 1)] * neighborWeights[3][1] +
      stateWeights[squareCell1D.getNeighbor(r - 1, c    )] * neighborWeights[3][2] +
      stateWeights[squareCell1D.getNeighbor(r - 1, c + 1)] * neighborWeights[3][3] +
      stateWeights[squareCell1D.getNeighbor(r - 1, c + 2)] * neighborWeights[3][4] +

      stateWeights[squareCell1D.getNeighbor(r    , c - 2)] * neighborWeights[4][0] +
      stateWeights[squareCell1D.getNeighbor(r    , c - 1)] * neighborWeights[4][1] +
      stateWeights[cell                                  ] * neighborWeights[4][2] +
      stateWeights[squareCell1D.getNeighbor(r    , c + 1)] * neighborWeights[4][3] +
      stateWeights[squareCell1D.getNeighbor(r    , c + 2)] * neighborWeights[4][4];
    if ((count >= 0) && (count < noCounts))
    {
      useTable[cell][count] = 1;
      return ruleTable[cell][count];
    }
    else
      return 0;
  }

  public int getNoStates()
  {
    return noStates;
  }

  public int getNoCounts()
  {
    return noCounts;
  }

  public int getPercentChange()
  {
    return percentChange;
  }

  public String getNewRuleOption()
  {
    return newRuleOption;
  }

  public boolean getClearNewRule()
  {
    return clearNewRule;
  }

  public int getStateWeight(int state)
  {
    return stateWeights[state];
  }

  public int getNeighborWeight(int row, int column)
  {
    return neighborWeights[row][column];
  }

  public int getRuleTableValue(int row, int column)
  {
    return ruleTable[row][column];
  }

  public int getUseTableValue(int row, int column)
  {
    return useTable[row][column];
  }

  public void setNoStates(int noStates)
  {
    this.noStates = noStates;
  }

  public void setNoCounts(int noCounts)
  {
    this.noCounts = noCounts;
  }

  public void setPercentChange(int percentChange)
  {
    this.percentChange = percentChange;
  }

  public void setNewRuleOption(String newRuleOption)
  {
    this.newRuleOption = newRuleOption;
  }

  public void setClearNewRule(boolean clearNewRule)
  {
    this.clearNewRule = clearNewRule;
  }

  public void setStateWeight(int state, int value)
  {
    stateWeights[state] = value;
  }

  public void setNeighborWeight(int row, int column, int value)
  {
    neighborWeights[row][column] = value;
  }

  public void setRuleTableValue(int row, int column, int value)
  {
    ruleTable[row][column] = value;
  }

  public void setUseTableValue(int row, int column, int value)
  {
    useTable[row][column] = value;
  }

  public boolean getRandomInitialization()
  {
    return randomInitialization;
  }

  public void setRandomInitialization(boolean randomInitialization)
  {
    this.randomInitialization = randomInitialization;
  }
 }
