import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.io.*;
import java.util.*;

public class Browser extends JDialog implements ActionListener, HyperlinkListener
{
  private String basePage;
  private String currentPage;
  private Vector<String> history;
  private boolean changed;

  private JEditorPane editorPane;
  private JScrollPane scrollPane;
  private JButton backButton;
  private JButton okButton;
  private JButton cancelButton;

  public Browser(JFrame owner, int width, int height, String baseURL, String currentURL)
  {
    super(owner, "Browser", true);
    basePage = baseURL;
    currentPage = currentURL;
		history = new Vector<String>();
		history.addElement(currentPage);
		editorPane = new JEditorPane();
		editorPane.setEditable(false);
		editorPane.addHyperlinkListener(this);
		setPage(editorPane, currentPage);
		scrollPane = new JScrollPane(editorPane);
    backButton = new JButton("Back");
    backButton.addActionListener(this);
    backButton.setEnabled(false);
    JPanel navigationPanel = new JPanel();
    navigationPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    navigationPanel.add(backButton);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    okButton.setEnabled(false);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    controlPanel.add(okButton);
    controlPanel.add(cancelButton);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new GridLayout(1, 2));
    buttonPanel.add(navigationPanel);
    buttonPanel.add(controlPanel);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(scrollPane, BorderLayout.CENTER);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setSize(width, height);
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

 	private void setPage(JEditorPane ep, String url)
  {
		try
    {
			ep.setPage(url);
		}
		catch (IOException e)
    {
      e.printStackTrace();
		}
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public String getCurrentPage()
  {
    return currentPage;
  }

  public void hyperlinkUpdate(HyperlinkEvent evt)
  {
    if (evt.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
    {
      String page = evt.getURL().toString();
      if (page.length() < basePage.length())
        return;
      if (page.substring(page.lastIndexOf('.')).equals(".sqc"))
        okButton.setEnabled(true);
      else
        okButton.setEnabled(false);
      currentPage = page;
      history.addElement(page);
      backButton.setEnabled(true);
      setPage(editorPane, page);
    }
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == backButton)
    {
      currentPage = history.lastElement();
      history.removeElement(currentPage);
      currentPage = history.lastElement();
      setPage(editorPane, currentPage);
      if (history.size() == 1)
        backButton.setEnabled(false);
    }
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}

