import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class CyclicOptionsDialog extends JDialog implements ActionListener, DocumentListener
{
  private int noCycles;
  private int noStates;
  private int noCounts;
  private int percentChange;
  private String newRuleOption;

  private boolean changed;

  private JLabel noCyclesLabel;
  private JLabel noStatesLabel;
  private JLabel noCountsLabel;
  private JTextField noCyclesInput;
  private JTextField noStatesInput;
  private JTextField noCountsInput;
  private JLabel percentChangeLabel;
  private JTextField percentChangeInput;
  private ButtonGroup newRuleOptionsSelector;
  private JRadioButton generationsButton;
  private JRadioButton modifiedGenerationsButton;
  private JRadioButton randomButton;
  private JCheckBox clearNewRuleButton;
  private JButton okButton;
  private JButton cancelButton;

  public CyclicOptionsDialog(JFrame owner)
  {
    super(owner, "Options", true);
    noCyclesLabel = new JLabel("No Cycles");
    noStatesLabel = new JLabel("No States");
    noCountsLabel = new JLabel("No Counts");
    noCyclesInput = new JTextField("" + noStates, 3);
    noCyclesInput.getDocument().addDocumentListener(this);
    noStatesInput = new JTextField("" + noStates, 3);
    noStatesInput.getDocument().addDocumentListener(this);
    noCountsInput = new JTextField("" + noCounts, 3);
    noCountsInput.getDocument().addDocumentListener(this);
    percentChangeLabel = new JLabel("% Change");
    percentChangeInput = new JTextField("" + percentChange, 3);
    percentChangeInput.getDocument().addDocumentListener(this);
    newRuleOptionsSelector = new ButtonGroup();
    generationsButton = new JRadioButton("Generations");
    generationsButton.addActionListener(this);
    newRuleOptionsSelector.add(generationsButton);
    modifiedGenerationsButton = new JRadioButton("Modified Generations");
    modifiedGenerationsButton.addActionListener(this);
    newRuleOptionsSelector.add(modifiedGenerationsButton);
    randomButton = new JRadioButton("Random");
    randomButton.addActionListener(this);
    newRuleOptionsSelector.add(randomButton);
    generationsButton.setSelected(true);
    clearNewRuleButton = new JCheckBox("Clear");
    clearNewRuleButton.setSelected(true);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel ruleParametersPanel = new JPanel();
    ruleParametersPanel.setLayout(new GridLayout(3, 2, 5, 5));
    JPanel noCyclesInputPanel = new JPanel();
    noCyclesInputPanel.add(noCyclesInput);
    JPanel noStatesInputPanel = new JPanel();
    noStatesInputPanel.add(noStatesInput);
    JPanel noCountsInputPanel = new JPanel();
    noCountsInputPanel.add(noCountsInput);
    ruleParametersPanel.add(noCyclesLabel);
    ruleParametersPanel.add(noCyclesInputPanel);
    ruleParametersPanel.add(noStatesLabel);
    ruleParametersPanel.add(noStatesInputPanel);
    ruleParametersPanel.add(noCountsLabel);
    ruleParametersPanel.add(noCountsInputPanel);
    ruleParametersPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Parameters"));
    JPanel percentChangePanel = new JPanel();
    percentChangePanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel percentChangeInputPanel = new JPanel();
    percentChangeInputPanel.add(percentChangeInput);
    percentChangePanel.add(percentChangeLabel);
    percentChangePanel.add(percentChangeInputPanel);
    JPanel controlButtonPanel = new JPanel();
    controlButtonPanel.setLayout(new GridLayout(4, 1, 5, 5));
    controlButtonPanel.add(generationsButton);
    controlButtonPanel.add(modifiedGenerationsButton);
    controlButtonPanel.add(randomButton);
    controlButtonPanel.add(clearNewRuleButton);
    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new BorderLayout(0, 5));
    controlPanel.add(percentChangePanel, BorderLayout.NORTH);
    controlPanel.add(controlButtonPanel, BorderLayout.SOUTH);
    controlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "New Rule Options"));
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(ruleParametersPanel, BorderLayout.NORTH);
    basePanel.add(controlPanel, BorderLayout.CENTER);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  private void setValues()
  {
    try
    {
      int y = new Integer(noCyclesInput.getText().trim());
      noCycles = y;
      if (noCycles < 1)
        noCycles = 1;
      int s = new Integer(noStatesInput.getText().trim());
      noStates = s;
      if (noStates < 2)
        noStates = 2;
      if (noStates > 16)
        noStates = 16;
      int c = new Integer(noCountsInput.getText().trim());
      noCounts = c;
      if (noCounts < 9)
        noCounts = 9;
      int p = new Integer(percentChangeInput.getText().trim());
      percentChange = p;
      if (percentChange < 1)
        percentChange = 1;
      if (percentChange > 100)
        percentChange = 100;
    }
    catch (Exception e)
    {
    }
  }

  public void setData(CyclicRule rule)
  {
    noCycles = rule.getNoCycles();
    noCyclesInput.setText("" + noCycles);
    noStates = rule.getNoStates();
    noStatesInput.setText("" + noStates);
    noCounts = rule.getNoCounts();
    noCountsInput.setText("" + noCounts);
    percentChange = rule.getPercentChange();
    percentChangeInput.setText("" + percentChange);
    newRuleOption = rule.getNewRuleOption();
    if (newRuleOption.equals("Generations"))
      generationsButton.setSelected(true);
    else if (newRuleOption.equals("Modified Generations"))
      modifiedGenerationsButton.setSelected(true);
    else if (newRuleOption.equals("Random"))
      randomButton.setSelected(true);
    clearNewRuleButton.setSelected(rule.getClearNewRule());
  }

  public void getData(CyclicRule rule)
  {
    rule.setNoCycles(noCycles);
    rule.setNoStates(noStates);
    rule.setNoCounts(noCounts);
    rule.setPercentChange(percentChange);
    rule.setNewRuleOption(newRuleOption);
    rule.setClearNewRule(clearNewRuleButton.isSelected());
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == generationsButton)
      newRuleOption = "Generations";
    else if (source == modifiedGenerationsButton)
      newRuleOption = "Modified Generations";
    else if (source == randomButton)
      newRuleOption = "Random";
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
