import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

class GeorgeMaydwellRule extends Rule
{
  private static final int maxNoStates = 16;
  private static final int ruleTableRows = 5;
  private static final int ruleTableColumns = 5;

  private int noStates;
  private int percentChange = 30;
  private String newRuleOption = "Random";
  private boolean clearNewRule = true;
  private int stateCounting[];
  private int ruleTable[][][];

  public GeorgeMaydwellRule(JFrame mainFrame, SquareCell squareCell)
  {
    super(mainFrame, squareCell);
    setDefault();
  }

  public void setNewRule()
  {
    if (clearNewRule)
    {
      for (int s = 0; s < noStates; s++)
        for (int r = 0; r < ruleTableRows; r++)
          for (int c = 0; c < ruleTableColumns; c++)
            ruleTable[s][r][c] = 0;
    }
    Random generator = new Random();
    if (newRuleOption.equals("Random"))
    {
      for (int s = 0; s < noStates; s++)
        for (int i = 1; i < ruleTableRows * ruleTableColumns * percentChange / 100; i++)
        {
          int r = generator.nextInt(ruleTableRows);
          int c = generator.nextInt(ruleTableColumns);
          ruleTable[s][r][c] = generator.nextInt(noStates);
        }
    }
    else if (newRuleOption.equals("Generations") || newRuleOption.equals("Modified Generations"))
    {
      for (int s = 1; s < (noStates - 1); s++)
        for (int r = 0; r < ruleTableRows; r++)
          for (int c = 0; c < ruleTableColumns; c++)
            ruleTable[s][r][c] = s + 1;
      if (newRuleOption.equals("Generations"))
        for (int s = 0; s < 2; s++)
          for (int i = 1; i < ruleTableRows * ruleTableColumns * percentChange / 100; i++)
          {
            int r = generator.nextInt(ruleTableRows);
            int c = generator.nextInt(ruleTableColumns);
            ruleTable[s][r][c] = 1;
          }
      else
        for (int s = 0; s < noStates; s++)
          for (int i = 1; i < ruleTableRows * ruleTableColumns * percentChange / 100; i++)
          {
            int r = generator.nextInt(ruleTableRows);
            int c = generator.nextInt(ruleTableColumns);
            ruleTable[s][r][c] = generator.nextInt(noStates);
          }
    }
  }

  public void setOptions()
  {
    int currentNoStates = noStates;
    GeorgeMaydwellOptionsDialog ruleTableOptionsDialog = new GeorgeMaydwellOptionsDialog(mainFrame);
    ruleTableOptionsDialog.setData(this);
    centerDialog(ruleTableOptionsDialog);
    ruleTableOptionsDialog.setVisible(true);
    if (ruleTableOptionsDialog.hasChanged())
    {
      ruleTableOptionsDialog.getData(this);
      if (noStates != currentNoStates)
        ruleTable = new int[noStates][ruleTableRows][ruleTableColumns];
    }
  }

  public void setDefault()
  {
    final int rt[][][] =
    {
      {
        {0,3,1,3,3},
        {3,1,3,3,3},
        {1,2,3,3,3},
        {2,3,3,3,3},
        {3,3,3,3,3}
      },
      {
        {2,2,2,2,2},
        {2,2,2,2,2},
        {2,2,2,2,2},
        {2,2,2,2,2},
        {2,2,2,2,2}
      },
      {
        {0,0,0,0,0},
        {0,0,0,0,0},
        {0,0,0,0,0},
        {0,0,0,0,0},
        {0,0,0,0,0}
      },
      {
        {3,4,1,4,4},
        {4,1,4,4,4},
        {1,2,4,4,4},
        {2,4,4,4,4},
        {4,4,4,4,4}
      },
      {
        {4,5,1,5,5},
        {5,1,5,5,5},
        {1,2,5,5,5},
        {2,5,5,5,5},
        {5,5,5,5,5}
      },
      {
        {5,6,1,6,6},
        {6,1,6,6,6},
        {1,2,6,6,6},
        {2,6,6,6,6},
        {6,6,6,6,6}
      },
      {
        {6,7,1,7,7},
        {7,1,7,7,7},
        {1,2,7,7,7},
        {2,7,7,7,7},
        {7,7,7,7,7}
      },
      {
        {7,8,1,8,8},
        {8,1,8,8,8},
        {1,2,8,8,8},
        {2,8,8,8,8},
        {8,8,8,8,8}
      },
      {
        {8,0,1,0,0},
        {0,1,0,0,0},
        {1,2,0,0,0},
        {2,0,0,0,0},
        {0,0,0,0,0}
      }
    };
    ruleTable = rt;
    stateCounting = new int[maxNoStates];
    for (int r = 0; r < maxNoStates; r++)
      stateCounting[r] = 0;
    stateCounting[1] = 3;
    noStates = 9;
  }

  public void changeRule()
  {
    GeorgeMaydwellDialog ruleTableDialog = new GeorgeMaydwellDialog(mainFrame, this, squareCell);
    ruleTableDialog.setData(this);
    centerDialog(ruleTableDialog);
    ruleTableDialog.setVisible(true);
    if (ruleTableDialog.hasChanged())
      ruleTableDialog.getData(this);
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  public void writeRule(PrintWriter pw)
  {
    pw.println("#Rule = George Maydwell Rule");
    pw.println("#States = " + noStates);
    String s = "#CT ";
    for (int i = 0; i < noStates; i++)
      s = s + stateCounting[i] + ",";
    pw.println(s.substring(0, s.lastIndexOf(",")));
    for (int i = 0; i < noStates; i++)
    {
      s = "#RT ";
      for (int j = 0; j < ruleTableRows; j++)
        for (int k = 0; k < ruleTableColumns; k++)
          s = s + ruleTable[i][j][k] + ",";
      pw.println(s.substring(0, s.lastIndexOf(",")));
    }
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      noStates = new Integer(br.readLine().substring(10).trim());
			for (int i = 0; i < maxNoStates; i++)
				stateCounting[i] = 0;
      String values[] = br.readLine().substring(3).split(",");
      for (int i = 0; i < noStates; i++)
        stateCounting[i] = new Integer(values[i].trim());
      ruleTable = new int[noStates][ruleTableRows][ruleTableColumns];
      for (int i = 0; i < noStates; i++)
      {
        values = br.readLine().substring(3).split(",");
        int s = 0;
        for (int j = 0; j < ruleTableRows; j++)
          for (int k = 0; k < ruleTableColumns; k++)
            ruleTable[i][j][k] = new Integer(values[s++].trim());
      }
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  private int getCount(int state, boolean adjacent)
  {
    int c = stateCounting[state];
    if (adjacent)
      if ((c & 1) > 0)
        return 1;
      else
        return 0;
    else
       if (((c >> 1) & 1) > 0)
        return 1;
      else
        return 0;
  }

  public int step(int r, int c)
  {
    int adjacentCount =
      getCount(squareCell.getNeighbor(r - 1, c    ), true) +
      getCount(squareCell.getNeighbor(r + 1, c    ), true) +
      getCount(squareCell.getNeighbor(r    , c - 1), true) +
      getCount(squareCell.getNeighbor(r    , c + 1), true);
    int diagonalCount =
      getCount(squareCell.getNeighbor(r - 1, c - 1), false) +
      getCount(squareCell.getNeighbor(r - 1, c + 1), false) +
      getCount(squareCell.getNeighbor(r + 1, c + 1), false) +
      getCount(squareCell.getNeighbor(r + 1, c - 1), false);
    return ruleTable[squareCell.getNeighbor(r, c)][adjacentCount][diagonalCount];
  }

  public int getStateCounting(int state)
  {
    return stateCounting[state];
  }

  public void setStateCounting(int state, int value)
  {
    stateCounting[state] = value;
  }

  public int[][][] getRuleTableData()
  {
    return ruleTable;
  }

  public void setRuleTableData(int[][][] ruleTableData)
  {
    ruleTable = ruleTableData;
  }

  public int getNoStates()
  {
    return noStates;
  }

  public int getPercentChange()
  {
    return percentChange;
  }

  public String getNewRuleOption()
  {
    return newRuleOption;
  }

  public boolean getClearNewRule()
  {
    return clearNewRule;
  }

  public void setNoStates(int noStates)
  {
    this.noStates = noStates;
  }

  public void setPercentChange(int percentChange)
  {
    this.percentChange = percentChange;
  }

  public void setNewRuleOption(String newRuleOption)
  {
    this.newRuleOption = newRuleOption;
  }

  public void setClearNewRule(boolean clearNewRule)
  {
    this.clearNewRule = clearNewRule;
  }
}
