import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

class InteractingStatesDialog extends JDialog implements ActionListener
{
  private static final int rows = 9;
  private static final int columns = 9;
  private InteractingStatesGrid birthControl;
  private InteractingStatesGrid surviveControl;
  private JButton okButton;
  private JButton cancelButton;
  private boolean changed = false;

  public InteractingStatesDialog(JFrame owner)
  {
    super(owner, "Interacting States", true);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout());
    JPanel gridPanel = new JPanel();
    birthControl = new InteractingStatesGrid(rows, columns, "Birth Control");
    gridPanel.add(birthControl);
    surviveControl = new InteractingStatesGrid(rows, columns, "Survival Control");
    gridPanel.add(surviveControl);
    basePanel.add(gridPanel, BorderLayout.CENTER);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
  }

  public void getData(InteractingStatesRule rule)
  {
    for (int r = 1; r < rows; r++)
      for (int c = 1; c < columns; c++)
      {
        rule.setBirth(r - 1, c - 1, birthControl.getValue(r, c));
        rule.setSurvive(r - 1, c - 1, surviveControl.getValue(r, c));
      }
    rule.setBirthRandomLimit(birthControl.getRandomLimit());
    rule.setSurviveRandomLimit(surviveControl.getRandomLimit());
    rule.setBirthOperatorSelector(birthControl.getOperatorSelector());
    rule.setSurviveOperatorSelector(surviveControl.getOperatorSelector());
  }

  public void setData(InteractingStatesRule rule)
  {
    for (int r = 1; r < rows; r++)
      for (int c = 1; c < columns; c++)
      {
        birthControl.setValue(r, c, rule.getBirth(r - 1, c - 1));
        surviveControl.setValue(r, c, rule.getSurvive(r - 1, c - 1));
      }
    birthControl.setRandomLimit(rule.getBirthRandomLimit());
    surviveControl.setRandomLimit(rule.getSurviveRandomLimit());
    birthControl.setOperatorSelector(rule.getBirthOperatorSelector());
    surviveControl.setOperatorSelector(rule.getSurviveOperatorSelector());
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}
