import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.*;

class InteractingStatesGrid extends JPanel implements ActionListener
{
  private static final String[] operatorSelectorStrings = {"OR >=", "OR =", "AND >=", "AND ="};
  
  private int rows;
  private int columns;
  private GridSpinner grid[][];
  private TextSpinner operatorSelectorSpinner;
  private JButton resetButton;
  private JButton randomButton;
  private GridSpinner randomLimit;
  private String operatorSelector;

  public InteractingStatesGrid(int rows, int columns, String title)
  {
    this.rows = rows;
    this.columns = columns;
    JPanel basePanel = new JPanel();
    basePanel.setLayout(new BorderLayout());
    JPanel gridPanel = new JPanel();
    gridPanel.setLayout(new GridLayout(rows, columns));
    grid = new GridSpinner[rows][columns];
    for (int r = 0; r < rows; r++)
      for (int c = 0; c < columns; c++)
      {
        boolean isLabel = true;
        String label = "";
        if ((r != 0) && (c != 0))
          isLabel = false;
        else if ((r == 0) && (c != 0))
          label = "" + c;
        else if ((r != 0) && (c == 0))
          label = "" + r;
        grid[r][c] = new GridSpinner(20, 20, label, 0, 0, 8, isLabel);
        gridPanel.add(grid[r][c]);
      }
    JPanel centerPanel = new JPanel();
    centerPanel.add(gridPanel);
    basePanel.add(centerPanel, BorderLayout.CENTER);
    JPanel buttonPanel = new JPanel();
    operatorSelectorSpinner = new TextSpinner(60, 20, operatorSelectorStrings);
    buttonPanel.add(operatorSelectorSpinner);
    resetButton = new JButton("Reset");
    resetButton.addActionListener(this);
    buttonPanel.add(resetButton);
    randomButton = new JButton("Random");
    randomButton.addActionListener(this);
    buttonPanel.add(randomButton);
    randomLimit = new GridSpinner(20, 20, "", 0, 0, 8, false);
    buttonPanel.add(randomLimit);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), title));
    add(basePanel);
  }

  public int getValue(int r, int c)
  {
    return grid[r][c].getValue();
  }

  public void setValue(int r, int c, int value)
  {
    grid[r][c].setValue(value);
  }

  public int getRandomLimit()
  {
    return randomLimit.getValue();
  }

  public void setRandomLimit(int value)
  {
    randomLimit.setValue(value);
  }

  public String getOperatorSelector()
  {
    return operatorSelectorSpinner.getText();
  }

  public void setOperatorSelector(String value)
  {
    for (int i = 0; i < operatorSelectorStrings.length; i++)
    {
      if (operatorSelectorStrings[i].equals(value))
      {
        operatorSelectorSpinner.setText(value);
        operatorSelectorSpinner.setSelectedIndex(i);
      }
    }
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == resetButton)
    {
      for (int r = 1; r < rows; r++)
        for (int c = 1; c < columns; c++)
          grid[r][c].setValue(0);
    }
    else if (source == randomButton)
    {
      Random generator = new Random();
      for (int r = 1; r < rows; r++)
      {
        grid[r][generator.nextInt(columns - 1) + 1].setValue(generator.nextInt(randomLimit.getValue() + 1));
        grid[r][generator.nextInt(columns - 1) + 1].setValue(generator.nextInt(randomLimit.getValue() + 1));
      }
    }
    operatorSelectorSpinner.requestFocus();
  }
}
