import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

class Counts2Dialog extends JDialog implements ActionListener
{

  private static final String[][] NeighborWeightsLabels =
                                           {
                                             {"NWNW", "NNW ", "NN  ", "NNE ", "NENE"},
                                             {"WNW ", "NW  ", "N   ", "NE  ", "ENE "},
                                             {"WW  ", "W   ", "C   ", "E   ", "EE  "},
                                             {"WSW ", "SW  ", "S   ", "SE  ", "ESE "},
                                             {"SWSW", "SSW ", "SS  ", "SSE ", "SESE"}
                                           };

  private static final int neighborRows = 5;
  private static final int neighborColumns = 5;
  private static final int minimumWeight = 0;
  private static final int maximumWeight = 2;

  private Counts2Rule rule;

  private int ruleTableSize;

  private WeightsGrid neighborWeightsGrid;
  private RuleTable ruleTable;
  private JButton randomButton;
  private JButton clearButton;
  private JButton okButton;
  private JButton cancelButton;
  private boolean changed = false;

  public Counts2Dialog(JFrame owner, Counts2Rule rule, SquareCell squareCell)
  {
    super(owner, "Counts 2", true);
    this.rule = rule;
    ruleTableSize = rule.getRuleTableSize();
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout());
    JPanel gridPanel = new JPanel();
    Border border = BorderFactory.createEtchedBorder();
    neighborWeightsGrid = new WeightsGrid(neighborRows, neighborColumns, border, "Neighbor Weights", NeighborWeightsLabels,
                                          minimumWeight, maximumWeight);
    gridPanel.add(neighborWeightsGrid);
    basePanel.add(gridPanel, BorderLayout.NORTH);
    JPanel ruleTablePanel = new JPanel();
    ruleTable = new RuleTable(ruleTableSize, ruleTableSize, squareCell, true);
    ruleTablePanel.add(ruleTable);
    ruleTablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Table"));
    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new GridLayout(2, 1, 5, 5));
    randomButton = new JButton("Random");
    randomButton.addActionListener(this);
    controlPanel.add(randomButton);
    clearButton = new JButton("Clear");
    clearButton.addActionListener(this);
    controlPanel.add(clearButton);
    ruleTablePanel.add(controlPanel);
    basePanel.add(ruleTablePanel, BorderLayout.CENTER);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
  }

  public void getData(Counts2Rule rule)
  {
    for (int r = 0; r < neighborRows; r++)
      for (int c = 0; c < neighborColumns; c++)
        rule.setNeighborWeight(r, c, neighborWeightsGrid.getValue(r, c));
    for (int r = 0; r < ruleTableSize; r++)
      for (int c = 0; c < ruleTableSize; c++)
        rule.setRuleTableValue(r, c, ruleTable.getState(r, c));
  }

  public void setData(Counts2Rule rule)
  {
    for (int r = 0; r < neighborRows; r++)
      for (int c = 0; c < neighborColumns; c++)
        neighborWeightsGrid.setValue(r, c, rule.getNeighborWeight(r, c));
    for (int r = 0; r < ruleTableSize; r++)
      for (int c = 0; c < ruleTableSize; c++)
        ruleTable.setState(r, c, rule.getRuleTableValue(r, c));
  }

  public void random()
  {
    rule.random(ruleTable.getGrid());
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == randomButton)
    {
      random();
      repaint();
    }
    else if (source == clearButton)
    {
      ruleTable.clear();
      repaint();
    }
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}

