import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class InteractingStatesOptionsDialog extends JDialog implements ActionListener, DocumentListener
{
  private int birthPercentChange;
  private int survivePercentChange;
  private boolean changed;

  private JCheckBox birthEnabledCheckBox;
  private JLabel birthPercentChangeLabel;
  private JTextField birthPercentChangeInput;
  private JCheckBox birthClearCheckBox;
  private JCheckBox surviveEnabledCheckBox;
  private JLabel survivePercentChangeLabel;
  private JTextField survivePercentChangeInput;
  private JCheckBox surviveClearCheckBox;
  private JButton okButton;
  private JButton cancelButton;

  public InteractingStatesOptionsDialog(JFrame owner)
  {
    super(owner, "Options", true);
    birthEnabledCheckBox = new JCheckBox("Enabled");
    birthEnabledCheckBox.setSelected(true);
    birthPercentChangeLabel = new JLabel("% Change");
    birthPercentChangeInput = new JTextField("" + birthPercentChange, 3);
    birthPercentChangeInput.getDocument().addDocumentListener(this);
    JPanel birthPercentChangePanel = new JPanel();
    birthPercentChangePanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel birthPercentChangeInputPanel = new JPanel();
    birthPercentChangeInputPanel.add(birthPercentChangeInput);
    birthPercentChangePanel.add(birthPercentChangeLabel);
    birthPercentChangePanel.add(birthPercentChangeInputPanel);
    birthClearCheckBox = new JCheckBox("Clear");
    birthClearCheckBox.setSelected(true);
    JPanel birthControlPanel = new JPanel();
    birthControlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Birth"));
    birthControlPanel.setLayout(new GridLayout(3, 1, 5, 5));
    birthControlPanel.add(birthEnabledCheckBox);
    birthControlPanel.add(birthPercentChangePanel);
    birthControlPanel.add(birthClearCheckBox);
    surviveEnabledCheckBox = new JCheckBox("Enabled");
    surviveEnabledCheckBox.setSelected(false);
    survivePercentChangeLabel = new JLabel("% Change");
    survivePercentChangeInput = new JTextField("" + survivePercentChange, 3);
    survivePercentChangeInput.getDocument().addDocumentListener(this);
    JPanel survivePercentChangePanel = new JPanel();
    survivePercentChangePanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel survivePercentChangeInputPanel = new JPanel();
    survivePercentChangeInputPanel.add(survivePercentChangeInput);
    survivePercentChangePanel.add(survivePercentChangeLabel);
    survivePercentChangePanel.add(survivePercentChangeInputPanel);
    surviveClearCheckBox = new JCheckBox("Clear");
    surviveClearCheckBox.setSelected(true);
    JPanel surviveControlPanel = new JPanel();
    surviveControlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Survive"));
    surviveControlPanel.setLayout(new GridLayout(3, 1, 5, 5));
    surviveControlPanel.add(surviveEnabledCheckBox);
    surviveControlPanel.add(survivePercentChangePanel);
    surviveControlPanel.add(surviveClearCheckBox);
    JPanel controlPanel = new JPanel();
    controlPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "New Rule Options"));
    controlPanel.setLayout(new GridLayout(2, 1, 5, 5));
    controlPanel.add(birthControlPanel);
    controlPanel.add(surviveControlPanel);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(controlPanel, BorderLayout.NORTH);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  private void setValues()
  {
    try
    {
      birthPercentChange = new Integer(birthPercentChangeInput.getText().trim());
      if (birthPercentChange < 1)
        birthPercentChange = 1;
      if (birthPercentChange > 100)
        birthPercentChange = 100;
      survivePercentChange = new Integer(survivePercentChangeInput.getText().trim());
      if (survivePercentChange < 1)
        survivePercentChange = 1;
      if (survivePercentChange > 100)
        survivePercentChange = 100;
    }
    catch (Exception e)
    {
    }
  }

  public void setData(InteractingStatesRule rule)
  {
    birthEnabledCheckBox.setSelected(rule.getBirthEnabled());
    surviveEnabledCheckBox.setSelected(rule.getSurviveEnabled());
    birthPercentChange = rule.getBirthPercentChange();
    birthPercentChangeInput.setText("" + birthPercentChange);
    survivePercentChange = rule.getSurvivePercentChange();
    survivePercentChangeInput.setText("" + survivePercentChange);
    birthClearCheckBox.setSelected(rule.getBirthClear());
    surviveClearCheckBox.setSelected(rule.getSurviveClear());
 }

  public void getData(InteractingStatesRule rule)
  {
    rule.setBirthEnabled(birthEnabledCheckBox.isSelected());
    rule.setSurviveEnabled(surviveEnabledCheckBox.isSelected());
    rule.setBirthPercentChange(birthPercentChange);
    rule.setSurvivePercentChange(survivePercentChange);
    rule.setBirthClear(birthClearCheckBox.isSelected());
    rule.setSurviveClear(surviveClearCheckBox.isSelected());
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
