import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

class MargolusDialog extends JDialog implements ActionListener
{
  private SquareCell squareCell;
  private MargolusRule rule;

  private RuleTable[] blockIndex;
  private RuleTable[] blockValue;
  private JPanel ruleTablePanel;
  private JButton okButton;
  private JButton cancelButton;
  private boolean changed = false;

  public MargolusDialog(JFrame owner, MargolusRule rule, SquareCell squareCell)
  {
    super(owner, "", true);
    this.squareCell = squareCell;
    this.rule = rule;
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout());
    Border border = BorderFactory.createEtchedBorder();
    ruleTablePanel = new JPanel();
    makeRuleTable();
    basePanel.add(ruleTablePanel, BorderLayout.CENTER);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
  }
  
  private void makeRuleTable()
  {
    ruleTablePanel.setLayout(new GridLayout(4, 4, 5, 5));
    blockIndex = new RuleTable[16];
    blockValue = new RuleTable[16];
    for (int i = 0; i < 16; i++)
    {
      makeBlockIndex(i);
      blockValue[i] = new RuleTable(2, 2, squareCell, true);
      JPanel indexPanel = new JPanel();
      indexPanel.add(blockIndex[i]);
      indexPanel.add(blockValue[i]);
      ruleTablePanel.add(indexPanel);
    }
    ruleTablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Table"));
  }

  private void makeBlockIndex(int index)
  {
    blockIndex[index] = new RuleTable(2, 2, squareCell, false);
    blockIndex[index].setState(0, 0, (index >> 3) & 1);
    blockIndex[index].setState(0, 1, (index >> 2) & 1);
    blockIndex[index].setState(1, 0, (index >> 1) & 1);
    blockIndex[index].setState(1, 1, index & 1);
  }

  public int getData(int i, int r, int c)
  {
    return blockValue[i].getState(r, c);
  }

  public void setData(int i, int r, int c, int v)
  {
    blockValue[i].setState(r, c, v);
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}
