import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

class MargolusRule extends Rule
{
  private SquareCell squareCell;
  private MargolusDialog margolusDialog;
  private int ruleTable[][][];
  private int cycle = 0;
  private int topLeftRow;
  private int topLeftColumn;
  
  public MargolusRule(JFrame mainFrame, SquareCell squareCell)
  {
    super(mainFrame, squareCell);
    this.squareCell = squareCell;
    ruleTable = new int[16][2][2];
  }

  public void setNewRule()
  {
    Random generator = new Random();
    for (int i = 0; i < 16; i++)
    {
      int v = generator.nextInt(16);
      ruleTable[i][0][0] = (v >> 3) & 1;
      ruleTable[i][0][1] = (v >> 2) & 1;
      ruleTable[i][1][0] = (v >> 1) & 1;
      ruleTable[i][1][1] = v & 1;
    }
  }

  public void setOptions()
  {
  }

  public void setDefault()
  {
  }

  public void changeRule()
  {
    margolusDialog = new MargolusDialog(mainFrame, this, squareCell);
    setData();
    centerDialog(margolusDialog);
    margolusDialog.setVisible(true);
    if (margolusDialog.hasChanged())
      getData();
  }

  private void centerDialog(JDialog dialog)
  {
    Point p = mainFrame.getLocation();
    int x = (int)p.getX() + (mainFrame.getWidth() - dialog.getWidth()) / 2;
    int y = (int)p.getY() + (mainFrame.getHeight() - dialog.getHeight()) / 2;
    dialog.setLocation(x, y);
  }

  private void getData()
  {
    for (int i = 0; i < 16; i++)
      for (int r = 0; r < 2; r++)
        for (int c = 0; c < 2; c++)
          ruleTable[i][r][c] = margolusDialog.getData(i, r, c);
  }

  private void setData()
  {
    for (int i = 0; i < 16; i++)
      for (int r = 0; r < 2; r++)
        for (int c = 0; c < 2; c++)
          margolusDialog.setData(i, r, c, ruleTable[i][r][c]);
  }

  public void writeRule(PrintWriter pw)
  {
    pw.println("#Rule = Margolus");
    pw.println("#Cycle = " + cycle);
    Selector selector = squareCell.getSelector();
    pw.println("#TopLeftRow = " + (selector.getTopLeftRow() & 1));
    pw.println("#TopLeftColumn = " + (selector.getTopLeftColumn() & 1));
    String s = "#RT ";
    for (int i = 0; i < 16; i++)
    {
      int v = ruleTable[i][0][0] * 8 + ruleTable[i][0][1] * 4 + ruleTable[i][1][0] * 2 + ruleTable[i][1][1];
      s = s + Integer.toHexString(v);
    }
    pw.println(s);
  }

  public void readRule(BufferedReader br)
  {
    try
    {
      cycle = new Integer(br.readLine().substring(9).trim());
      topLeftRow = new Integer(br.readLine().substring(14).trim());
      topLeftColumn = new Integer(br.readLine().substring(17).trim());
      String s = br.readLine();
      for (int i = 0; i <16; i++)
        fromHex(i, s.substring(i + 4, i + 5));
    }
    catch (IOException e)
    {
      e.printStackTrace();
    }
  }

  private void fromHex(int i, String s)
  {
    int v = Integer.parseInt(s, 16);
    ruleTable[i][0][0] = (v >> 3) & 1;
    ruleTable[i][0][1] = (v >> 2) & 1;
    ruleTable[i][1][0] = (v >> 1) & 1;
    ruleTable[i][1][1] = v & 1;
  }

  public void step(byte[][] cells, int rows, int columns)
  {
    if (cycle == 0)
    {
      for (int r = 0; r < rows; r += 2)
        for (int c = 0; c < columns; c += 2)
        {
          int r1 = r + 1;
          int c1 = c + 1;
          int i = cells[r][c] * 8 + cells[r][c1] * 4 + cells[r1][c] * 2 + cells[r1][c1];
          cells[r][c] = (byte) ruleTable[i][0][0];
          cells[r][c1] = (byte) ruleTable[i][0][1];
          cells[r1][c] = (byte) ruleTable[i][1][0];
          cells[r1][c1] = (byte) ruleTable[i][1][1];
        }
    }
    else
    {
      for (int r = 1; r < rows; r += 2)
        for (int c = 1; c < columns; c += 2)
        {
          int r1 = (r + 1) % rows;
          int c1 = (c + 1) % columns;
          int i = cells[r][c] * 8 + cells[r][c1] * 4 + cells[r1][c] * 2 + cells[r1][c1];
          cells[r][c] = (byte) ruleTable[i][0][0];
          cells[r][c1] = (byte) ruleTable[i][0][1];
          cells[r1][c] = (byte) ruleTable[i][1][0];
          cells[r1][c1] = (byte) ruleTable[i][1][1];
        }
    }
    cycle = (cycle + 1) % 2;
  }

  public int getNoStates()
  {
    return 2;
  }

  public int getCycle()
  {
    return cycle;
  }

  public void setCycle(int cycle)
  {
    this.cycle = cycle;
  }

  public int getTopLeftRow()
  {
    return topLeftRow;
  }

  public int getTopLeftColumn()
  {
    return topLeftColumn;
  }
}
