import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

class NeighborButton extends JPanel implements MouseListener
{
  private final int squareSize = 6;
  private final int border = 4;

  private int value;
  private int grid[][];
  private boolean selected;
  private BufferedImage image;
  private ColorSelector colorSelector;
  
  public NeighborButton(int value, SquareCell squareCell)
  {
    this.value = value;
    this.colorSelector = squareCell.getColorSelector();
    grid = new int[3][3];
    grid[2][2] = value & 1;
    grid[2][1] = (value >> 1) & 1;
    grid[2][0] = (value >> 2) & 1;
    grid[1][0] = (value >> 3) & 1;
    grid[0][0] = (value >> 4) & 1;
    grid[0][1] = (value >> 5) & 1;
    grid[0][2] = (value >> 6) & 1;
    grid[1][2] = (value >> 7) & 1;
    grid[1][1] = 0;
    selected = false;
    setOpaque(true);
    setDoubleBuffered(false);
    setPreferredSize(new Dimension(3 * squareSize + 2 * border, 3 * squareSize + 2 * border));
    addMouseListener(this);
  }

  public void paintComponent(Graphics g)
  {
    super.paintComponent(g);
    image = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_RGB);
    drawSquares();
    g.drawImage(image, 0, 0, null);
  }

  private void drawSquares()
  {
    Graphics g = image.getGraphics();
    if (selected)
      g.setColor(colorSelector.getColor(2));
    else
      g.setColor(colorSelector.getColor(3));
    g.fillRect(0, 0, image.getWidth(), image.getHeight());
    for (int r = 0; r < 3; r++)
      for (int c = 0; c < 3; c++)
        drawSquare(g, r, c, colorSelector.getColor(grid[r][c]));
    g.dispose();
  }

  private void drawSquare(Graphics g, int r, int c, Color color)
  {
    int y = border + (r * squareSize);
    int h = squareSize;
    int x = border + (c * squareSize);
    int w = squareSize;
    g.setColor(colorSelector.getGridColor());
    g.drawRect(x, y, squareSize, squareSize);
    g.setColor(color);
    g.fillRect(x + 1, y + 1, squareSize - 1, squareSize - 1);
  }

  public int getValue()
  {
    return value;
  }

  public boolean getSelected()
  {
    return selected;
  }

  public void setSelected(boolean s)
  {
    selected = s;
  }

  public void mousePressed(MouseEvent e)
  {
    selected = !selected;
    repaint();
  }

  public void mouseExited(MouseEvent e) {}
  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
}
