import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class NewWorld extends JDialog implements ActionListener, DocumentListener
{
  private int rows;
  private int columns;
  private String selectedRule;

  private boolean changed;

  private JLabel rowsLabel;
  private JLabel columnsLabel;
  private JTextField rowsInput;
  private JTextField columnsInput;
  private ButtonGroup ruleSelector;
  private JRadioButton ruleTableButton;
  private JRadioButton ruleTable2Button;
  private JRadioButton weightedGenerationsButton;
  private JRadioButton interactingStatesButton;
  private JRadioButton countsButton;
  private JRadioButton counts2Button;
  private JRadioButton bitCountsButton;
  private JRadioButton cyclicButton;
  private JRadioButton margolusButton;
  private JRadioButton GeorgeMaydwellButton;
  private JRadioButton NonTotalisticButton;
  private JRadioButton particleStreamsButton;
  private JRadioButton diagonalParticleStreamsButton;
  private JRadioButton knightShipsButton;
  private JButton okButton;
  private JButton cancelButton;

  public NewWorld(JFrame owner, int rows, int columns, String selectedRule)
  {
    super(owner, "New World", true);
    this.rows = rows;
    this.columns = columns;
    this.selectedRule = selectedRule;
    rowsLabel = new JLabel("Rows");
    columnsLabel = new JLabel("Columns");
    rowsInput = new JTextField("" + rows, 4);
    rowsInput.getDocument().addDocumentListener(this);
    columnsInput = new JTextField("" + columns, 4);
    columnsInput.getDocument().addDocumentListener(this);
    ruleSelector = new ButtonGroup();
    ruleTableButton = new JRadioButton("Rule Table");
    ruleTableButton.addActionListener(this);
    ruleSelector.add(ruleTableButton);
    ruleTable2Button = new JRadioButton("Rule Table 2");
    ruleTable2Button.addActionListener(this);
    ruleSelector.add(ruleTable2Button);
    weightedGenerationsButton = new JRadioButton("Weighted Generations");
    weightedGenerationsButton.addActionListener(this);
    ruleSelector.add(weightedGenerationsButton);
    interactingStatesButton = new JRadioButton("Interacting States");
    interactingStatesButton.addActionListener(this);
    ruleSelector.add(interactingStatesButton);
    countsButton = new JRadioButton("Counts");
    countsButton.addActionListener(this);
    ruleSelector.add(countsButton);
    counts2Button = new JRadioButton("Counts 2");
    counts2Button.addActionListener(this);
    ruleSelector.add(counts2Button);
    bitCountsButton = new JRadioButton("Bit Counts");
    bitCountsButton.addActionListener(this);
    ruleSelector.add(bitCountsButton);
    cyclicButton = new JRadioButton("Cyclic");
    cyclicButton.addActionListener(this);
    ruleSelector.add(cyclicButton);
    margolusButton = new JRadioButton("Margolus");
    margolusButton.addActionListener(this);
    ruleSelector.add(margolusButton);
    GeorgeMaydwellButton = new JRadioButton("George Maydwell");
    GeorgeMaydwellButton.addActionListener(this);
    ruleSelector.add(GeorgeMaydwellButton);
    NonTotalisticButton = new JRadioButton("Non Totalistic");
    NonTotalisticButton.addActionListener(this);
    ruleSelector.add(NonTotalisticButton);
    particleStreamsButton = new JRadioButton("Particle Streams");
    particleStreamsButton.addActionListener(this);
    ruleSelector.add(particleStreamsButton);
    diagonalParticleStreamsButton = new JRadioButton("Diagonal Particle Streams");
    diagonalParticleStreamsButton.addActionListener(this);
    ruleSelector.add(diagonalParticleStreamsButton);
    knightShipsButton = new JRadioButton("Knight Ships");
    knightShipsButton.addActionListener(this);
    ruleSelector.add(knightShipsButton);
    if (selectedRule.equals("Rule Table"))
      ruleTableButton.setSelected(true);
    else if (selectedRule.equals("Rule Table 2"))
      ruleTable2Button.setSelected(true);
    else if (selectedRule.equals("Weighted Generations"))
      weightedGenerationsButton.setSelected(true);
    else if (selectedRule.equals("Interacting States"))
      interactingStatesButton.setSelected(true);
    else if (selectedRule.equals("Counts"))
      countsButton.setSelected(true);
    else if (selectedRule.equals("Counts 2"))
      counts2Button.setSelected(true);
    else if (selectedRule.equals("Bit Counts"))
      bitCountsButton.setSelected(true);
    else if (selectedRule.equals("Cyclic"))
      cyclicButton.setSelected(true);
    else if (selectedRule.equals("Margolus"))
      margolusButton.setSelected(true);
    else if (selectedRule.equals("George Maydwell Rule"))
      GeorgeMaydwellButton.setSelected(true);
    else if (selectedRule.equals("Non Totalistic Rule"))
      NonTotalisticButton.setSelected(true);
    else if (selectedRule.equals("Particle Streams"))
      particleStreamsButton.setSelected(true);
    else if (selectedRule.equals("Diagonal Particle Streams"))
      diagonalParticleStreamsButton.setSelected(true);
    else if (selectedRule.equals("Knight Ships"))
      knightShipsButton.setSelected(true);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    JPanel worldSizePanel = new JPanel();
    worldSizePanel.setLayout(new GridLayout(2, 2, 5, 5));
    JPanel rowsInputPanel = new JPanel();
    rowsInputPanel.add(rowsInput);
    JPanel columnsInputPanel = new JPanel();
    columnsInputPanel.add(columnsInput);
    worldSizePanel.add(rowsLabel);
    worldSizePanel.add(rowsInputPanel);
    worldSizePanel.add(columnsLabel);
    worldSizePanel.add(columnsInputPanel);
    worldSizePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "World Size"));
    JPanel ruleSelectorPanel = new JPanel();
    ruleSelectorPanel.setLayout(new GridLayout(7, 2, 5, 5));
    ruleSelectorPanel.add(ruleTableButton);
    ruleSelectorPanel.add(ruleTable2Button);
    ruleSelectorPanel.add(weightedGenerationsButton);
    ruleSelectorPanel.add(interactingStatesButton);
    ruleSelectorPanel.add(countsButton);
    ruleSelectorPanel.add(counts2Button);
    ruleSelectorPanel.add(bitCountsButton);
    ruleSelectorPanel.add(cyclicButton);
    ruleSelectorPanel.add(margolusButton);
    ruleSelectorPanel.add(GeorgeMaydwellButton);
    ruleSelectorPanel.add(NonTotalisticButton);
    ruleSelectorPanel.add(particleStreamsButton);
    ruleSelectorPanel.add(diagonalParticleStreamsButton);
    ruleSelectorPanel.add(knightShipsButton);
    ruleSelectorPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Selector"));
    JPanel optionsPanel = new JPanel();
    optionsPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 10, 5));
    optionsPanel.add(worldSizePanel);
    optionsPanel.add(ruleSelectorPanel);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.setLayout(new BorderLayout(0, 5));
    basePanel.add(optionsPanel, BorderLayout.NORTH);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
    setValues();
  }

  private void setValues()
  {
    try
    {
      int r = new Integer(rowsInput.getText().trim());
      int c = new Integer(columnsInput.getText().trim());
      rows = r;
      if (rows < 10)
        rows = 10;
      columns = c;
      if (columns < 10)
        columns = 10;
    }
    catch (Exception e)
    {
    }
  }

  public int getRows()
  {
    return rows;
  }

  public int getColumns()
  {
    return columns;
  }

  public String getSelectedRule()
  {
    return selectedRule;
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == ruleTableButton)
      selectedRule = "Rule Table";
    else if (source == ruleTable2Button)
      selectedRule = "Rule Table 2";
    else if (source ==  weightedGenerationsButton)
      selectedRule = "Weighted Generations";
    else if (source == interactingStatesButton)
      selectedRule = "Interacting States";
    else if (source == countsButton)
      selectedRule = "Counts";
    else if (source == counts2Button)
      selectedRule = "Counts 2";
    else if (source == bitCountsButton)
      selectedRule = "Bit Counts";
    else if (source == cyclicButton)
      selectedRule = "Cyclic";
    else if (source == margolusButton)
    {
      selectedRule = "Margolus";
      if ((rows % 2) == 1)
        rows++;
      if ((columns % 2) == 1)
        columns++;
    }
    else if (source == GeorgeMaydwellButton)
      selectedRule = "George Maydwell Rule";
    else if (source == NonTotalisticButton)
      selectedRule = "Non Totalistic Rule";
    else if (source == particleStreamsButton)
      selectedRule = "Particle Streams";
    else if (source == diagonalParticleStreamsButton)
      selectedRule = "Diagonal Particle Streams";
    else if (source == knightShipsButton)
      selectedRule = "Knight Ships";
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
