import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

class NonTotalisticDialog extends JDialog implements ActionListener
{
  private static final int neighborhood[][] = NonTotalisticRule.neighborhood;

  private NonTotalisticRule rule;

  private NeighborButton[] birthNeighborButtons;
  private NeighborButton[] surviveNeighborButtons;

  private JTextField ruleInput;
  private JButton makeRuleButton;
  private JButton pasteButton;
  private JButton okButton;
  private JButton cancelButton;
  private boolean changed = false;
  
  private int[] ruleTable;

  public NonTotalisticDialog(JFrame owner, NonTotalisticRule rule, SquareCell squareCell)
  {
    super(owner, "", true);
    setTitle("Non Totalistic");
    this.rule = rule;
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout());

    JPanel birthPanel = new JPanel();
    birthPanel.setLayout(new GridLayout(9, 1, 2, 2));
    birthPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Birth"));
    birthNeighborButtons = new NeighborButton[70];
    JPanel birth0Panel = new JPanel();
    birth0Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    birthNeighborButtons[0] = new NeighborButton(neighborhood[0][0], squareCell);
    birth0Panel.add(birthNeighborButtons[0]);
    JPanel birth1Panel = new JPanel();
    birth1Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 1; r < 3; r++)
    {
      birthNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      birth1Panel.add(birthNeighborButtons[r]);
    }
    JPanel birth2Panel = new JPanel();
    birth2Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 3; r < 11; r++)
    {
      birthNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      birth2Panel.add(birthNeighborButtons[r]);
    }
    JPanel birth3Panel = new JPanel();
    birth3Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 11; r < 25; r++)
    {
      birthNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      birth3Panel.add(birthNeighborButtons[r]);
    }
    JPanel birth4Panel = new JPanel();
    birth4Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 25; r < 45; r++)
    {
      birthNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      birth4Panel.add(birthNeighborButtons[r]);
    }
    JPanel birth5Panel = new JPanel();
    birth5Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 45; r < 59; r++)
    {
      birthNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      birth5Panel.add(birthNeighborButtons[r]);
    }
    JPanel birth6Panel = new JPanel();
    birth6Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 59; r < 67; r++)
    {
      birthNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      birth6Panel.add(birthNeighborButtons[r]);
    }
    JPanel birth7Panel = new JPanel();
    birth7Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 67; r < 69; r++)
    {
      birthNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      birth7Panel.add(birthNeighborButtons[r]);
    }
    JPanel birth8Panel = new JPanel();
    birth8Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    birthNeighborButtons[69] = new NeighborButton(neighborhood[69][0], squareCell);
    birth8Panel.add(birthNeighborButtons[69]);
    birthPanel.add(birth0Panel);
    birthPanel.add(birth1Panel);
    birthPanel.add(birth2Panel);
    birthPanel.add(birth3Panel);
    birthPanel.add(birth4Panel);
    birthPanel.add(birth5Panel);
    birthPanel.add(birth6Panel);
    birthPanel.add(birth7Panel);
    birthPanel.add(birth8Panel);
    basePanel.add(birthPanel, BorderLayout.NORTH);

    JPanel survivePanel = new JPanel();
    survivePanel.setLayout(new GridLayout(9, 1, 2, 2));
    survivePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Survive"));
    surviveNeighborButtons = new NeighborButton[70];
    JPanel survive0Panel = new JPanel();
    survive0Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    surviveNeighborButtons[0] = new NeighborButton(neighborhood[0][0], squareCell);
    survive0Panel.add(surviveNeighborButtons[0]);
    JPanel survive1Panel = new JPanel();
    survive1Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 1; r < 3; r++)
    {
      surviveNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      survive1Panel.add(surviveNeighborButtons[r]);
    }
    JPanel survive2Panel = new JPanel();
    survive2Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 3; r < 11; r++)
    {
      surviveNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      survive2Panel.add(surviveNeighborButtons[r]);
    }
    JPanel survive3Panel = new JPanel();
    survive3Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 11; r < 25; r++)
    {
      surviveNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      survive3Panel.add(surviveNeighborButtons[r]);
    }
    JPanel survive4Panel = new JPanel();
    survive4Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 25; r < 45; r++)
    {
      surviveNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      survive4Panel.add(surviveNeighborButtons[r]);
    }
    JPanel survive5Panel = new JPanel();
    survive5Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 45; r < 59; r++)
    {
      surviveNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      survive5Panel.add(surviveNeighborButtons[r]);
    }
    JPanel survive6Panel = new JPanel();
    survive6Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 59; r < 67; r++)
    {
      surviveNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      survive6Panel.add(surviveNeighborButtons[r]);
    }
    JPanel survive7Panel = new JPanel();
    survive7Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    for (int r = 67; r < 69; r++)
    {
      surviveNeighborButtons[r] = new NeighborButton(neighborhood[r][0], squareCell);
      survive7Panel.add(surviveNeighborButtons[r]);
    }
    JPanel survive8Panel = new JPanel();
    survive8Panel.setLayout(new FlowLayout(FlowLayout.LEFT, 2, 2));
    surviveNeighborButtons[69] = new NeighborButton(neighborhood[69][0], squareCell);
    survive8Panel.add(surviveNeighborButtons[69]);
    survivePanel.add(survive0Panel);
    survivePanel.add(survive1Panel);
    survivePanel.add(survive2Panel);
    survivePanel.add(survive3Panel);
    survivePanel.add(survive4Panel);
    survivePanel.add(survive5Panel);
    survivePanel.add(survive6Panel);
    survivePanel.add(survive7Panel);
    survivePanel.add(survive8Panel);
    basePanel.add(survivePanel, BorderLayout.CENTER);

    JPanel bottomPanel = new JPanel();
    bottomPanel.setLayout(new BorderLayout());
    JPanel makeRulePanel = new JPanel();
    makeRulePanel.setLayout(new GridLayout(2, 1, 2, 2));
    makeRulePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "New Rule"));
    ruleInput = new JTextField("");
    makeRulePanel.add(ruleInput);
    JPanel ruleButtonPanel = new JPanel();
    makeRuleButton = new JButton("Make Rule");
    makeRuleButton.addActionListener(this);
    ruleButtonPanel.add(makeRuleButton);
    pasteButton = new JButton("Paste");
    pasteButton.addActionListener(this);
    ruleButtonPanel.add(pasteButton);
    makeRulePanel.add(ruleButtonPanel);
    bottomPanel.add(makeRulePanel, BorderLayout.NORTH);

    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    bottomPanel.add(buttonPanel, BorderLayout.SOUTH);

    basePanel.add(bottomPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
  }

  private void makeRuleFromString(String s, NeighborButton[] b)
  {
    String stringOffsets[] =
    {
      "0", "ce", "acakevki", "ijvaqcvqjyryke", "avwvatjkyzriqycrkzje", "avyqjiekjyrqvc", "aekikacv", "ec", "8"
    };
    String so = "";
    int offsets[] = {0, 1, 3, 11, 25, 45, 59, 67, 69};
    int index;
    int offset = 0;
    
    if (s.equals(""))
      return;
    for (int i = 0; i < 70; i++)
      b[i].setSelected(false);
    while (s.length() > 0)
    {
      String t = s.substring(0, 1);
      s = s.substring(1);
      if (t.equals("0"))
        b[0].setSelected(true);
      else if (t.equals("8"))
        b[69].setSelected(true);
      else if ((t.compareTo("0") > 0) && (t.compareTo("8") < 0))
      {
        index = Integer.parseInt(t);
        offset = offsets[index];
        so = stringOffsets[index];
        if ((s.length() == 0) || ((s.substring(0, 1).compareTo("0") >= 0) && (s.substring(0, 1).compareTo("8") <= 0)))
        {
          for (int i = 0; i < so.length(); i++)
            b[offset + i].setSelected(true);
        }
      }
      else
        for (int i = 0; i < so.length(); i++)
          if (t.equals(so.substring(i, i + 1)))
            b[offset + i].setSelected(true);
    }
  }

  private void makeRuleTable()
  {
    ruleTable = new int[256];
    for (int i = 0; i < 256; i++)
      ruleTable[i] = 0;
    for (int i = 0; i < 70; i++)
    {
      int birth = 0;
      if (birthNeighborButtons[i].getSelected())
        birth = 2;
      int survive = 0;
      if (surviveNeighborButtons[i].getSelected())
        survive = 1;
      for (int j = 0; j < 4; j++)
        ruleTable[neighborhood[i][j]] = birth + survive;
    }
  }

  public void getData(NonTotalisticRule rule)
  {
    makeRuleTable();
    rule.setRuleTable(ruleTable);
  }

  public void setData(NonTotalisticRule rule)
  {
    int[] rt = rule.getRuleTable();
    for (int i = 0; i < 70; i++)
    {
      if ((rt[birthNeighborButtons[i].getValue()] & 2) > 0)
        birthNeighborButtons[i].setSelected(true);
      else
        birthNeighborButtons[i].setSelected(false);
      if ((rt[surviveNeighborButtons[i].getValue()] & 1) > 0)
        surviveNeighborButtons[i].setSelected(true);
      else
        surviveNeighborButtons[i].setSelected(false);
    }
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == makeRuleButton)
    {
      String s = ruleInput.getText();
      if (!s.substring(0, 1).equals("s"))
      {
        ruleInput.setText("Enter rule 's../b..'");
        return;
      }
      int i = s.indexOf("/");
      String survive = s.substring(1, i);
      s = s.substring(i + 1);
      if (!s.substring(0, 1).equals("b"))
      {
        ruleInput.setText("Enter rule 's../b..'");
        return;
      }
      String birth = s.substring(1);
      makeRuleFromString(birth, birthNeighborButtons);
      makeRuleFromString(survive, surviveNeighborButtons);
      repaint();
    }
    else if (source == pasteButton)
      ruleInput.paste();
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}