import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

public class NonTotalisticOptionsDialog extends JDialog implements ActionListener, DocumentListener
{
  private int percentChange;
  private String symmetry;

  private boolean changed;

  private JLabel percentChangeLabel;
  private JTextField percentChangeInput;
  private JCheckBox clearNewRuleButton;
  private ButtonGroup symmetrySelector;
  private JRadioButton fullButton;
  private JRadioButton rotationalButton;
  private JRadioButton noneButton;
  private JButton okButton;
  private JButton cancelButton;

  public NonTotalisticOptionsDialog(JFrame owner)
  {
    super(owner, "Options", true);
    percentChangeLabel = new JLabel("% Change");
    percentChangeInput = new JTextField("" + percentChange, 3);
    percentChangeInput.getDocument().addDocumentListener(this);
    symmetrySelector = new ButtonGroup();
    fullButton = new JRadioButton("Full");
    fullButton.addActionListener(this);
    symmetrySelector.add(fullButton);
    rotationalButton = new JRadioButton("Rotational");
    rotationalButton.addActionListener(this);
    symmetrySelector.add(rotationalButton);
    noneButton = new JRadioButton("None");
    noneButton.addActionListener(this);
    symmetrySelector.add(noneButton);
    fullButton.setSelected(true);
    clearNewRuleButton = new JCheckBox("Clear");
    clearNewRuleButton.setSelected(true);
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "New Rule Options"));
    basePanel.setLayout(new BorderLayout(0, 5));
    JPanel controlPanel = new JPanel();
    controlPanel.setLayout(new GridLayout(2, 1, 5, 5));
    JPanel percentChangePanel = new JPanel();
    percentChangePanel.setLayout(new GridLayout(1, 2, 5, 5));
    JPanel percentChangeInputPanel = new JPanel();
    percentChangeInputPanel.add(percentChangeInput);
    percentChangePanel.add(percentChangeLabel);
    percentChangePanel.add(percentChangeInputPanel);
    controlPanel.add(percentChangePanel);
    controlPanel.add(clearNewRuleButton);
    JPanel symmetryPanel = new JPanel();
    symmetryPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Symmetry"));
    symmetryPanel.setLayout(new GridLayout(3, 1, 5, 5));
    symmetryPanel.add(fullButton);
    symmetryPanel.add(rotationalButton);
    symmetryPanel.add(noneButton);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    basePanel.add(controlPanel, BorderLayout.NORTH);
    basePanel.add(symmetryPanel, BorderLayout.CENTER);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
    changed = false;
  }

  private void setValues()
  {
    try
    {
      int p = new Integer(percentChangeInput.getText().trim());
      percentChange = p;
      if (percentChange < 1)
        percentChange = 1;
      if (percentChange > 100)
        percentChange = 100;
    }
    catch (Exception e)
    {
    }
  }

  public void setData(NonTotalisticRule rule)
  {
    percentChange = rule.getPercentChange();
    percentChangeInput.setText("" + percentChange);
    clearNewRuleButton.setSelected(rule.getClearNewRule());
    symmetry = rule.getSymmetry();
    if (symmetry.equals("Full"))
      fullButton.setSelected(true);
    else if (symmetry.equals("Rotational"))
      rotationalButton.setSelected(true);
    else if (symmetry.equals("None"))
      noneButton.setSelected(true);
  }

  public void getData(NonTotalisticRule rule)
  {
    rule.setPercentChange(percentChange);
    rule.setClearNewRule(clearNewRuleButton.isSelected());
    rule.setSymmetry(symmetry);
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == fullButton)
      symmetry = "Full";
    else if (source == rotationalButton)
      symmetry = "Rotational";
    else if (source == noneButton)
      symmetry = "None";
    else if (source == okButton)
    {
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }

  public void insertUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void removeUpdate(DocumentEvent event)
  {
    setValues();
  }
  
  public void changedUpdate(DocumentEvent event) {}
}
