import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

class GeorgeMaydwellDialog extends JDialog implements ActionListener
{
  private static final String[][] StateLabels =
                                       {
                                         {"S0 ", "S8 "},
                                         {"S1 ", "S9 "},
                                         {"S2 ", "S10"},
                                         {"S3 ", "S11"},
                                         {"S4 ", "S12"},
                                         {"S5 ", "S13"},
                                         {"S6 ", "S14"},
                                         {"S7 ", "S15"}
                                       };

  private static final int stateRows = 8;
  private static final int stateColumns = 2;
  private static final int lowGridValue = 0;
  private static final int highGridValue = 3;
  private static final int ruleTableRows = 7;
  private static final int ruleTableColumns = 7;

  private WeightsGrid stateCountingGrid;
  private int ruleTableData[][][];
  private int noStates;
  private int currentState;
  private GeorgeMaydwellRule rule;
  private RuleTable ruleTable;
  private JButton ruleTableSelector;
  private JButton okButton;
  private JButton cancelButton;
  private boolean changed = false;

  public GeorgeMaydwellDialog(JFrame owner, GeorgeMaydwellRule rule, TriangularCell triangularCell)
  {
    super(owner, "", true);
    setTitle("Rule Table");
    this.rule = rule;
    JPanel basePanel = new JPanel();
    basePanel.setBorder(BorderFactory.createLineBorder(basePanel.getBackground(), 15));
    basePanel.setLayout(new BorderLayout());
    JPanel gridPanel = new JPanel();
    Border border = BorderFactory.createEtchedBorder();
    stateCountingGrid = new WeightsGrid(stateRows, stateColumns, border, "State Counting", StateLabels,
                                        lowGridValue, highGridValue);
    gridPanel.add(stateCountingGrid);
    basePanel.add(gridPanel, BorderLayout.NORTH);
    JPanel ruleTablePanel = new JPanel();
    ruleTablePanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Rule Table"));
    ruleTable = new RuleTable(ruleTableRows, ruleTableColumns, triangularCell, true);
    ruleTableSelector = new JButton(currentState + "");
    ruleTableSelector.addActionListener(this);
    ruleTablePanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    ruleTablePanel.add(ruleTable);
    ruleTablePanel.add(ruleTableSelector);
    basePanel.add(ruleTablePanel,BorderLayout.CENTER);
    JPanel buttonPanel = new JPanel();
    buttonPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
    okButton = new JButton("Ok");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    cancelButton = new JButton("Cancel");
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    basePanel.add(buttonPanel, BorderLayout.SOUTH);
    add(basePanel);
    pack();
    setResizable(false);
    basePanel.requestFocus();
  }

  public void getData(GeorgeMaydwellRule rule)
  {
		for (int i = 0; i < noStates; i++)
			rule.setStateCounting(i, stateCountingGrid.getValue(i % stateRows, i / stateRows));
    rule.setRuleTableData(ruleTableData);
  }

  public void setData(GeorgeMaydwellRule rule)
  {
    noStates = rule.getNoStates();
    for (int r = 0; r < stateRows; r++)
      for (int c = 0; c < stateColumns; c++)
        stateCountingGrid.setValue(r, c, rule.getStateCounting(c * stateRows + r));
    int rt[][][] = rule.getRuleTableData();
    ruleTableData = new int[noStates][ruleTableRows][ruleTableColumns];
    for (int s = 0; s < noStates; s++)
      for (int r = 0; r < ruleTableRows; r++)
        for (int c = 0; c < ruleTableColumns; c++)
          ruleTableData[s][r][c] = rt[s][r][c];
    currentState = 0;
    for (int r = 0; r < ruleTableRows; r++)
      for(int c = 0; c < ruleTableColumns; c++)
        ruleTable.setState(r, c, ruleTableData[currentState][r][c]);
  }

  public boolean hasChanged()
  {
    return changed;
  }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();
    if (source == ruleTableSelector)
    {
      for (int r = 0; r < ruleTableRows; r++)
        for(int c = 0; c < ruleTableColumns; c++)
          ruleTableData[currentState][r][c] = ruleTable.getState(r, c);
      currentState = (currentState + 1) % rule.getNoStates();
      ruleTableSelector.setText(currentState + "");
      for (int r = 0; r < ruleTableRows; r++)
        for(int c = 0; c < ruleTableColumns; c++)
          ruleTable.setState(r, c, ruleTableData[currentState][r][c]);
      repaint();
    }
    else if (source == okButton)
    {
      for (int r = 0; r < ruleTableRows; r++)
        for(int c = 0; c < ruleTableColumns; c++)
          ruleTableData[currentState][r][c] = ruleTable.getState(r, c);
      setVisible(false);
      changed = true;
    }
    else if (source == cancelButton)
    {
      setVisible(false);
      changed = false;
    }
  }
}
