import java.awt.*;
import java.awt.geom.*;

class TriGridSpinner
{
  private NeighborGrid neighborGrid;
  private int x;
  private int y;
  private int spinnerWidth;
  private int spinnerHeight;
  private boolean pointingUp;
  private Polygon polygon;
  private int weight;

  public TriGridSpinner(int x, int y, int width, int height, boolean pointingUp, int weight)
  {
    this.neighborGrid = neighborGrid;
    this.x = x;
    this.y = y;
    spinnerWidth = width;
    spinnerHeight = height;
    this.pointingUp = pointingUp;
    this.weight = weight;
  }

  private Polygon getPolygon(int x, int y, boolean pointingUp)
  {
    int x1 = x + spinnerWidth / 2;
    int y1 = y + spinnerHeight;
    int x2 = x + spinnerWidth;
    polygon = new Polygon();
    if (pointingUp)
    {
      polygon.addPoint(x, y1);
      polygon.addPoint(x1, y);
      polygon.addPoint(x2, y1);
    }
    else
    {
      polygon.addPoint(x, y);
      polygon.addPoint(x2, y);
      polygon.addPoint(x1, y1);
    }
    return polygon;
  }

  private void drawWeight(Graphics g)
  {
    FontMetrics fm = g.getFontMetrics();
    String s = "" + weight;
    Rectangle2D bounds = fm.getStringBounds(s, g);
    int xOffset = x + (spinnerWidth - (int)bounds.getWidth()) / 2;
    int yOffset = y + spinnerHeight / 2;
    if (pointingUp)
      yOffset += (int)bounds.getHeight();
      g.drawString(s, xOffset, yOffset);
  }

  public void draw(Graphics g) 
  {
    if (polygon == null)
      polygon = getPolygon(x, y, pointingUp);
    g.setColor(Color.black);
    g.drawPolygon(polygon);
    drawWeight(g);
  }

  public boolean contains(int x, int y)
  {
    return polygon.contains(x, y);
  }

  public int getWeight()
  {
    return weight;
  }

  public void setWeight(int weight)
  {
    this.weight = weight;
  }
}